% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% areagraph (): show employment decision of firm as a function of (z,n).
%
% Instructions: simply run 'areagraph()' without arguments
%
% ========================================================================


function areagraph
% ----
% Init

pdeg=4; % degree of polynomial approximation
warning off;

Nz=1000;
Nn=25;


% -------------------------
% Computing equilibrium

% Here, I recompute the model in the baseline calibration for greater
% precision along (z,n) while less precision along (y,s) -- to skip, load
% areagraph.mat directly

fprintf('Computing equilibrium...\n');

param = struct('A',1,'alpha',.85,'b',1.402502411917219,'beta',.996,'c',1.789395096004241,'ke',14.213281945105606,'kf',1.956256938229172,'lambda',0.366041621306308,...
    'rho_y',0.990372641907069,'sigma_y',0.042251089970073,'rho_z',(.95)^(1/3),'sigma_z',0.532628971768815,...
    'rho_s',0.98,'sigma_s',0.131668241312217,'cor_ys',-0.4,...
    'gamma',1.59929,'m',1,...
    'ny',3,'ns',5,'nz',50,'nn',150);  % these parameters correspond to calibration.mat but with different grids


vars = initialize_vars(param,1,1,2,2,3,3);
Y0_id=ceil(param.ny/2);
S0_id=ceil(param.ns/2);


% compute GE
[res,aux,error]=solve(param,vars,[],[],'GE');

% compute PE
res_PE = res;
res_PE.cost = res.cost(Y0_id + (S0_id-1)*param.ny)*ones(param.ny*param.ns,1);
[res_PE,aux_PE,error]=solve(param,vars,res_PE,[],'PE');

save('results/areagraph.mat');
% load('results/impulse.mat');

% ------------------------
% Compute optimal policies


zaxis=linspace(vars.zmin,vars.zmax,Nz);
naxis=linspace(vars.nmin,vars.nmax,Nn);

% general equilibrium versions
[draw_hi_unfit,draw_sep_unfit,draw_exit_unfit,draw_lf_unfit,draw_hi,draw_sep,draw_exit,draw_lf]=find_areas(param,vars,res,zaxis,naxis,Y0_id,S0_id,pdeg);
[draw_hi_y_unfit,draw_sep_y_unfit,draw_exit_y_unfit,draw_lf_y_unfit,draw_hi_y,draw_sep_y,draw_exit_y,draw_lf_y]=find_areas(param,vars,res,zaxis,naxis,Y0_id-1,S0_id,pdeg);
[draw_hi_s_unfit,draw_sep_s_unfit,draw_exit_s_unfit,draw_lf_s_unfit,draw_hi_s,draw_sep_s,draw_exit_s,draw_lf_s]=find_areas(param,vars,res,zaxis,naxis,Y0_id,S0_id+1,pdeg);
% partial equilibrium versions
[draw_hi_PE_unfit,draw_sep_PE_unfit,draw_exit_PE_unfit,draw_lf_PE_unfit,draw_hi_PE,draw_sep_PE,draw_exit_PE,draw_lf_PE]=find_areas(param,vars,res_PE,zaxis,naxis,Y0_id,S0_id,pdeg);
[draw_hi_y_PE_unfit,draw_sep_y_PE_unfit,draw_exit_y_PE_unfit,draw_lf_y_PE_unfit,draw_hi_y_PE,draw_sep_y_PE,draw_exit_y_PE,draw_lf_y_PE]=find_areas(param,vars,res_PE,zaxis,naxis,Y0_id-1,S0_id,pdeg);
[draw_hi_s_PE_unfit,draw_sep_s_PE_unfit,draw_exit_s_PE_unfit,draw_lf_s_PE_unfit,draw_hi_s_PE,draw_sep_s_PE,draw_exit_s_PE,draw_lf_s_PE]=find_areas(param,vars,res_PE,zaxis,naxis,Y0_id,S0_id+1,pdeg);


% ------------
% Plot results

linewidth=1;

close all;
figure('Units','inches','Position',[0,0,5,3],'PaperPositionMode','auto');
set(gcf,'DefaultAxesColorOrder',[0 0 0]);
set(gcf,'Name','Boundaries');
set(gcf,'DefaultAxesColorOrder',[0 0 0]);
hold on
plot(draw_exit_unfit,naxis,'LineWidth',1);
plot(draw_hi_unfit,naxis,'LineWidth',1);
plot(draw_sep_unfit,naxis,'LineWidth',1);
plot(draw_lf,naxis,'--','LineWidth',1);
xlabel('Idiosyncratic productivity z','FontUnits','points','interpreter','latex','FontWeight','normal','FontSize',9,'FontName','Times');
ylabel('Size n','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
ylim([0,70]);
xlim([0 1.75]);
text(.14,55,'Exit','interpreter','latex','FontUnits','points','FontWeight','normal','FontSize',12,'FontName','Times');
text(.54,45,'Separation','interpreter','latex','FontUnits','points','FontWeight','normal','FontSize',12,'FontName','Times');
text(.55,40,'layoffs+quits','interpreter','latex','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
text(.68,60,'quits only','interpreter','latex','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
text(1.05,35,'Inaction','interpreter','latex','FontUnits','points','FontWeight','normal','FontSize',12,'FontName','Times');
text(1.4,15,'Hiring','interpreter','latex','FontUnits','points','FontWeight','normal','FontSize',12,'FontName','Times');
text(1.40,45,'hiring threshold','rotation',81,'FontUnits','points','interpreter','latex','FontSize',8,'FontName','Times','Color',[0 .25 .75]);
text(1.12,48,'sep. threshold','rotation',78,'FontUnits','points','interpreter','latex','FontSize',8,'FontName','Times','Color',[0 .25 .75]);
text(.32,15,'exit threshold','rotation',90,'FontUnits','points','interpreter','latex','FontSize',8,'FontName','Times','Color',[0 .25 .75]);
set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
box on;
% arrows
pos = get(gca,'Position');
x = [.95,1.11]; % arrow for quits only
y = [60,62];
ar=annotation('arrow',x/1.75*pos(3)+pos(1),y/70*pos(4)+pos(2));
set(ar,'HeadLength',2,'HeadWidth',2,'HeadStyle','ellipse');
hold off
% export eps
print('-depsc2','graphs/area.eps');


% Plot with productivity shock
figure('Units','inches','Position',[0,0,5,7.5],'PaperPositionMode','auto');
set(gcf,'DefaultAxesColorOrder',[0 0 0],'Name','Productivity shock');

% Plot with productivity shock - PE
subplot(2,1,1);
hold on
plot(draw_exit_PE_unfit,naxis,'LineWidth',linewidth);
plot(draw_exit_y_PE_unfit,naxis,'--','LineWidth',linewidth,'Color',[0 .25 .75]);
plot(draw_hi_PE_unfit,naxis,'LineWidth',linewidth);
plot(draw_hi_y_PE_unfit,naxis,'--','LineWidth',linewidth,'Color',[0 .25 .75]);
plot(draw_sep_PE_unfit,naxis,'LineWidth',linewidth);
plot(draw_sep_y_PE_unfit,naxis,'--','LineWidth',linewidth,'Color',[0 .25 .75]);
% plot(draw_lf_PE_unfit,naxis,'LineWidth',linewidth,'Color',[1 0 0]);
% plot(draw_lf_y_PE_unfit,naxis,'--','LineWidth',linewidth,'Color',[1 0 0]);
xlabel('Idiosyncratic productivity z','FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
ylabel('Size n','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
ylim([0,70]);
xlim([0 1.75]);
title('(a) Partial Equilibrium','FontUnits','points','interpreter','latex','FontSize',12,'FontName','Times');
text(.14,55,'Exit','FontUnits','points','interpreter','latex','FontSize',10,'FontName','Times');
text(.6,45,'Separation','FontUnits','points','interpreter','latex','FontSize',10,'FontName','Times');
text(1.08,35,'Inaction','FontUnits','points','interpreter','latex','FontSize',10,'FontName','Times');
text(1.4,15,'Hiring','FontUnits','points','interpreter','latex','FontSize',10,'FontName','Times');
set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');


box on;
hold off


% Plot with productivity shock - GE
subplot(2,1,2);
hold on
plot(draw_exit_unfit,naxis,'LineWidth',linewidth);
plot(draw_exit_y_unfit,naxis,'--','LineWidth',linewidth,'Color',[0 .25 .75]);
plot(draw_hi_unfit,naxis,'LineWidth',linewidth);
plot(draw_hi_y_unfit,naxis,'--','LineWidth',linewidth,'Color',[0 .25 .75]);
plot(draw_sep_unfit,naxis,'LineWidth',linewidth);
plot(draw_sep_y_unfit,naxis,'--','LineWidth',linewidth,'Color',[0 .25 .75]);
% plot(draw_lf_unfit,naxis,'LineWidth',linewidth,'Color',[1 0 0]);
% plot(draw_lf_y_unfit,naxis,'--','LineWidth',linewidth,'Color',[1 0 0]);
xlabel('Idiosyncratic productivity z','FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
ylabel('Size n','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
ylim([0,70]);
xlim([0 1.75]);
title('(b) General Equilibrium','FontUnits','points','interpreter','latex','FontSize',12,'FontName','Times');
set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
box on;
hold off

print('-depsc2','graphs/area_y.eps');


% Plot with volatility shock
hdl=figure;
set(hdl,'Units','inches','Position',[0,0,5,7.5],'PaperPositionMode','auto');
set(hdl,'DefaultAxesColorOrder',[0 0 0],'Name','Volatility shock');

% Plot with volatility shock - PE
subplot(2,1,1);
hold on
plot(draw_exit_PE_unfit,naxis,'LineWidth',linewidth);
plot(draw_exit_s_PE_unfit,naxis,'--','LineWidth',linewidth,'Color',[0 .25 .75]);
plot(draw_hi_PE_unfit,naxis,'LineWidth',linewidth);
plot(draw_hi_s_PE_unfit,naxis,'--','LineWidth',linewidth,'Color',[0 .25 .75]);
plot(draw_sep_PE_unfit,naxis,'LineWidth',linewidth);
plot(draw_sep_s_PE_unfit,naxis,'--','LineWidth',linewidth,'Color',[0 .25 .75]);
xlabel('Idiosyncratic productivity z','FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
ylabel('Size n','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
ylim([0,70]);
xlim([0 1.75]);
title('(a) Partial Equilibrium','FontUnits','points','interpreter','latex','FontSize',12,'FontName','Times');
text(.19,55,'Exit','FontUnits','points','interpreter','latex','FontSize',10,'FontName','Times');
text(.55,45,'Separation','FontUnits','points','interpreter','latex','FontSize',10,'FontName','Times');
text(1.05,35,'Inaction','FontUnits','points','interpreter','latex','FontSize',10,'FontName','Times');
text(1.4,15,'Hiring','FontUnits','points','interpreter','latex','FontSize',10,'FontName','Times');
set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
box on;
hold off



% Plot with volatility shock - GE - presentation
subplot(2,1,2);
hold on
plot(draw_exit_unfit,naxis,'LineWidth',linewidth);
plot(draw_exit_s_unfit,naxis,'--','LineWidth',linewidth,'Color',[0 .25 .75]);
plot(draw_hi_unfit,naxis,'LineWidth',linewidth);
plot(draw_hi_s_unfit,naxis,'--','LineWidth',linewidth,'Color',[0 .25 .75]);
plot(draw_sep_unfit,naxis,'LineWidth',linewidth);
plot(draw_sep_s_unfit,naxis,'--','LineWidth',linewidth,'Color',[0 .25 .75]);
xlabel('Idiosyncratic productivity z','FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
ylabel('Size n','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
ylim([0,70]);
xlim([0 1.75]);
title('(b) General Equilibrium','FontUnits','points','interpreter','latex','FontSize',12,'FontName','Times');
set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
box on;
hold off

print('-depsc2','graphs/area_v.eps');

end


function [draw_hi,draw_sep,draw_exit,draw_lf,draw_hi_fit,draw_sep_fit,draw_exit_fit,draw_lf_fit]=find_areas(param,vars,res,zaxis,naxis,Y0_id,S0_id,pdeg)
% Init

Y0=vars.y_grid(Y0_id);
S0=vars.s_grid(S0_id);

Nz = length(zaxis);
Nn = length(naxis);

% -------------------------
% Retrieve optimal policies
M = reshape(res.M,param.ny,param.ns,param.nz,param.nn,param.nn); % dimension (y,s,z,n,nhat)
M = squeeze(M(Y0_id,S0_id,:,:,:));

[Mz_id Mn_id Mnhat_id]=ndgrid(1:param.nz,1:param.nn,1:param.nn);
V = M(Mz_id(:,:,1)+(Mn_id(:,:,1)-1)*param.nz + (Mn_id(:,:,1)-1)*param.nz*param.nn); % retrieve value of inaction

M_spline=griddedInterpolant(vars.z_grid(Mz_id),vars.n_grid(Mn_id),vars.n_grid(Mnhat_id),M);
V_spline=griddedInterpolant(vars.z_grid(Mz_id(:,:,1)),vars.n_grid(Mn_id(:,:,1)),V);


% ------------------
% Find optimal areas

[Mz Mn Mnhat]=ndgrid(zaxis,naxis,naxis);

Vval = V_spline(Mz(:,:,1),Mn(:,:,1)); % value of inaction

% Separation
Mnhat_sep=min(Mnhat,Mn);
Msep = M_spline(Mz,Mn,Mnhat_sep);
Msep = max(Msep,[],3);
sep = Msep > Vval;
draw_sep=zeros(1,Nn);

% Hiring
Mnhat_hi=max(Mnhat,Mn);
Mhi = M_spline(Mz,Mn,Mnhat_hi);
Mhi = max(Mhi,[],3);
hi = Mhi > Vval;
draw_hi=zeros(1,Nn);

% Exit
Mz=Mz(:,:,1);
Mn=Mn(:,:,1);
exit=res.exit_value_spline(Y0*ones(Nz,Nn),S0*ones(Nz,Nn),Mz,Mn) >= res.cont_value_spline(Y0*ones(Nz,Nn),S0*ones(Nz,Nn),Mz,Mn);
draw_exit=zeros(1,Nn);

% Layoffs
lf=res.gtau_spline(Y0*ones(Nz,Nn),S0*ones(Nz,Nn),Mz,Mn)>0;
draw_lf=zeros(1,Nn);

% --------------------
% Find area boundaries


for n=1:Nn
    I=find(exit(:,n)==1);
    if(isempty(I))
        I=1;
    end
    draw_exit(n)=zaxis(max(I));
    
    I=find(lf(:,n)==1);
    if(isempty(I))
        I=1;
    end
    draw_lf(n)=zaxis(max(I));
    draw_lf(n)=max(draw_lf(n),draw_exit(n)); % include exits in layoffs
    
    
    I=find(sep(:,n)==1);
    if(isempty(I))
        I=1;
    end
    draw_sep(n)=zaxis(max(I));
    if naxis(n)==0
        draw_sep(1) = zaxis(1); % get rid of spurious effects when n=0, impose sep=0
    end
    draw_sep(n)=max(draw_sep(n),draw_exit(n)); % block the separation decision from crossing layoff line (due to spline approximations)
    
    
    I=find(hi(:,n)==1);
    if(isempty(I))
        I=Nz;
    end
    draw_hi(n)=zaxis(min(I));
    if naxis(n)==vars.nmax
        draw_hi(n) = zaxis(end); % get rid of spurious effects when n=nmax, impose hi=0
    end
    draw_hi(n)=max(draw_hi(n),draw_sep(n)); % block the hiring decision from crossing separation line
    
    
end

% fit polynomials for boundaries
p_hi=polyfit(naxis,draw_hi,pdeg);
draw_hi_fit=polyval(p_hi,naxis);

p_sep=polyfit(naxis,draw_sep,pdeg);
draw_sep_fit=polyval(p_sep,naxis);

p_exit=polyfit(naxis,draw_exit,pdeg);
draw_exit_fit=polyval(p_exit,naxis);

% the layoffs zone is not smooth because of the grid, so I fit a polynomial
% in the part within the separation zone
draw_lf_fit=zeros(Nn,1); 
I=find(draw_lf>draw_sep);
ni=max(I);
p_lf=polyfit(naxis(ni+1:end-1),draw_lf(ni+1:end-1),pdeg); % remove last point which is off
draw_lf_fit(ni+1:end)=polyval(p_lf,naxis(ni+1:end));
draw_lf_fit(1:ni)=draw_sep(1:ni); %when the curve is not 'exactly' within the separation zone because of numerical errors, set equal to separation zone

end

