% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% calibrate (): script that calibrates the model by fitting the 12 targets 
% described in the paper. It solves a numerical distance minimization
% problem. In practice, this may take a few days/weeks. So the results have
% been saved in 'results/calibration.mat'.
%
% Instructions: simply run the script 'calibrate()' without arguments
%
% ========================================================================

function [param,vars,res,aux,error]=calibrate

% ------------------------------------
% Declare and initialize parameters
%

param = struct('A',1,'alpha',.85,'b',1.402502411917219,'beta',.996,'c',1.789395096004241,'ke',14.213281945105606,'kf',1.956256938229172,'lambda',0.366041621306308,...
        'rho_y',0.990372641907069,'sigma_y',0.042251089970073,'rho_z',(.95)^(1/3),'sigma_z',0.532628971768815,...
        'rho_s',0.98,'sigma_s',0.131668241312217,'cor_ys',-0.4,...
        'gamma',1.59929,'m',1,...
        'ny',3,'ns',3,'nz',15,'nn',50); % initialize with my own calibration results
    
vars = initialize_vars(param);

% Moments are [rho_output,sigma_output,IQR,rho_IQR,sigma_IQR,cor_IQR_output,UE,EU,EE,elas_UE_theta,avgfirmsize,entry/JC]
targets=[.839,.017,.393,.76,.049,-.092,.45,.026,.029,.72,15.6,.21];

% Draws a time series of (y,s)
T=50; % number of years
lag=1; % max number of lags in years
draws=[vars.y_grid(ceil(param.ny/2));vars.s_grid(ceil(param.ns/2))]*ones(1,(T+lag)*12);
rng(0,'twister');
for t=2:(T+lag)*12
    [draws(1,t),draws(2,t)]=draw_ys_value (draws(1,t-1),draws(2,t-1),param,vars);
end

% ---------------
% Initialization
%
start_time=clock();
date_string = datestr(now,'mm-dd-yy.HHMM');
delete('calibration.log');
diary('calibration.log');
fprintf('\n\n------------------------------------------------------------------------------\n');
fprintf('Calibration started on %s.\n',datestr(now,'mm-dd-yy at HH:MM'));

% Start calibration

settings=optimset('Display','iter','DiffMinChange',0.01);
[X,FVAL,EXITFLAG,OUTPUT]=fminsearch(@(x) objective_fun(x,param,vars,[],[],targets,draws),[log(param.b),log(param.c),log(param.ke),log(param.kf),log(param.lambda),tan(pi/2*param.rho_y),log(param.sigma_y),tan(pi/2*param.cor_ys),log(param.sigma_z),log(param.sigma_s),tan(pi/2*param.rho_s),log(param.gamma)],settings);
end_time=clock();

% Return results
if EXITFLAG>=0
    param.b=exp(X(1));
    param.c=exp(X(2));
    param.ke=exp(X(3));
    param.kf=exp(X(4));
    param.lambda=exp(X(5));
    param.rho_y=2/pi*atan(X(6));
    param.sigma_y=exp(X(7));
    param.cor_ys=2/pi*atan(X(8));
    param.sigma_z=exp(X(9));
    param.sigma_s=exp(X(10));
    param.rho_s=2/pi*atan(X(11));
    param.gamma=exp(X(12));
    
    
    [res,aux,error]=solve(param,vars,[],[]);
        
    format long;
    fprintf('\n\n------------------------------------------------------------------------------\n');
    fprintf('Calibration SUCCESS after %5.1f seconds and %d iterations!\n',etime(end_time,start_time),OUTPUT.funcCount);
    fprintf('------------------------------------------------------------------------------\n');
    fprintf('Final calibration results:\n\n');         
    fprintf('                 b                 c                ke                kf            lambda             gamma\n');
    disp([param.b param.c param.ke param.kf param.lambda param.gamma]);
    fprintf('             rho_y           sigma_y            cor_ys           sigma_z           sigma_s             rho_s\n');
    disp([param.rho_y param.sigma_y param.cor_ys param.sigma_z param.sigma_s param.rho_s]);
    
    
    [moments,sim_error]=compute_moments(param,vars,res,draws);
    
    score=score_fun(moments,targets);
    fprintf('Score=%d\n',score);    
    msg='SUCCESS';
else    
    fprintf('\n\n------------------------------------------------------------------------------\n');
    fprintf('Calibration FAILURE after %5.1f seconds and %d iterations!\n',etime(end_time,start_time),OUTPUT.funcCount);
    fprintf('------------------------------------------------------------------------------\n');
    fprintf('Error code=%d\n',error);
    msg='FAILURE';
end

diary off;

end

function val=objective_fun(x,param,vars,res,aux,targets,draws)
    param.b=exp(x(1));
    param.c=exp(x(2));
    param.ke=exp(x(3));
    param.kf=exp(x(4));
    param.lambda=exp(x(5));
    param.rho_y=2/pi*atan(x(6));
    param.sigma_y=exp(x(7));
    param.cor_ys=2/pi*atan(x(8));
    param.sigma_z=exp(x(9));
    param.sigma_s=exp(x(10));
    param.rho_s=2/pi*atan(x(11));
    param.gamma=exp(x(12));
    
    format long;
    fprintf('\n------------------------------------------------------------------------------\n');
    fprintf('\nEvaluating model for parameters:\n');
    fprintf('                 b                 c                ke                kf            lambda             gamma\n');
    disp([param.b param.c param.ke param.kf param.lambda param.gamma]);
    fprintf('             rho_y           sigma_y            cor_ys           sigma_z           sigma_s             rho_s\n');
    disp([param.rho_y param.sigma_y param.cor_ys param.sigma_z param.sigma_s param.rho_s]);
            
    
    [res,aux,error]=solve(param,vars,res,aux);
    
    % Error codes
    % 1 - V<ke
    % 2 - V(0)>ke
    % 4 - solvetheta.m reached max iterations
    % 8 - solveU.m reached max iterations
    % 16 - solve.m reached max iterations    
    % 32 - compute_moments.m reached max iterations    
    
    if error
        val=inf;                
    else
        [moments,sim_error]=compute_moments(param,vars,res,draws);
        if sim_error
            error=error+sim_error*32;
            val=inf;
        else
            val=score_fun(moments,targets);        
        end
    end
    fprintf('\nScore=%d, error code=%d\n',val,error);
end

function score=score_fun(moments,targets)
   tolerance=[2e-3,5e-1,2e-2,2e-2,2e-3,1e-2,5e-3,5e-2,2e-2,4e-2,2e-2,2e-2];
   score=mean(abs(moments-targets)./tolerance); 
end
