% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% compute_moments (param,vars,res,draws): 
% computes and displays the 12 moments targeted in the calibration.
%
% Instructions: - 'param' is a structure of parameters for the model
% - 'vars' is a structure of variables created by 'initialize_vars(param)'
% - 'res' is a structure of results for the model from 'solve(param,vars)'
% - 'draws' is a Tx2 matrix of y and s shock passed as arguments because 
% they are kept constant in the calibration process.
%
% ========================================================================

function [moments,error]=compute_moments(param,vars,res,draws)

T=50; % number of years
lag=1; % max number of lags in years
Y0=ceil(param.ny/2);
S0=ceil(param.ns/2);

if nargin<4 % draws not specified
    % Draw a time series of (y,s)
    draws=[vars.y_grid(Y0);vars.s_grid(S0)]*ones(1,(T+lag)*12);
    rng(0,'twister');
    for t=2:(T+lag)*12
        [draws(1,t),draws(2,t)]=draw_ys_value (draws(1,t-1),draws(2,t-1),param,vars);        
    end
end

[u0,g0]=findss(Y0,S0,param,vars,res); % find ss when y is constant equal to mean


% ----------------
% Start simulation


fprintf('\nComputing statistics ');
time0=clock();

[unemp,iqr_y,std_y,outpp,v,theta,ue,eu,ee,entry,exit,hi,sp,qu,lf,output,error,avgfrmsize] = simulate(draws(1,:),draws(2,:),param,vars,res,u0,g0);


% Format simulated series
%


% Get rid of the first 12 months with incomplete history
unemp=unemp(lag*12+1:(lag+T)*12);
outpp=outpp(lag*12+1:(lag+T)*12);
output=output(lag*12+1:(lag+T)*12);
v=v(lag*12+1:(lag+T)*12);
theta=theta(lag*12+1:(lag+T)*12);
ue=ue(lag*12+1:(lag+T)*12);
eu=eu(lag*12+1:(lag+T)*12);
ee=ee(lag*12+1:(lag+T)*12);
entry=entry(lag*12+1:(lag+T)*12);
exit=exit(lag*12+1:(lag+T)*12);
hi=hi(lag*12+1:(lag+T)*12);
sp=sp(lag*12+1:(lag+T)*12);
qu=qu(lag*12+1:(lag+T)*12);
lf=lf(lag*12+1:(lag+T)*12);


% Compute quarterly averages
unemp_q=averageM2Q(unemp);
outpp_q=averageM2Q(outpp);
output_q=averageM2Q(output);
theta_q=averageM2Q(theta);
v_q=averageM2Q(v);
ue_q=averageM2Q(ue);
eu_q=averageM2Q(eu);
ee_q=averageM2Q(ee);
entry_q=averageM2Q(entry);
exit_q=averageM2Q(exit);
hi_q=averageM2Q(hi);
sp_q=averageM2Q(sp);
qu_q=averageM2Q(qu);
lf_q=averageM2Q(lf);

% Compute log deviations
logue_dev=logdev(ue_q,'Q');
logtheta_dev=logdev(theta_q,'Q');

% Yearly average
output_y = averageQ2Y(output_q);
logoutput_y_dev = logdev(output_y,'Y');

logiqr_dev=logdev(iqr_y,'Y');

% Compute iqr statistics
logoutput_dev = logdev(output_q,'Q'); 
std_output = std(logoutput_dev);
rho_output = autocorr(logoutput_dev,1);
rho_output=rho_output(2);
cor_output_iqr=corr(logoutput_y_dev,logiqr_dev);

rho_iqr = autocorr(iqr_y,1); % careful the moments I have for IQR are not on logiqr_dev!
rho_iqr=rho_iqr(2);

% Compute elasticity of job finding rate with respect to theta
breg=regress(logue_dev,[ones(numel(logue_dev),1) logtheta_dev]);
e_UE_theta=breg(2);


% ------------------------------
% Display results
%

fprintf('completed in %3.1f seconds.\n',etime(clock(),time0));
format short;

fprintf('       (1)       (2)       (3)       (4)       (5)       (6)       (7)       (8)       (9)      (10)      (11)      (12)\n');
fprintf('    rho(Y)    std(Y)  Avg(IQR)  Rho(IQR)  Std(IQR) cr(Y,IQR)        UE        EU        EE  e_UE_th.  firmsize  entry/hi\n');
fprintf('    (.839)    (.016)    (.393)    (0.76)   (0.049)   (-.092)    (.450)    (.026)    (.029)     (.72)    (15.6)    (.210)\n');
disp([rho_output,std_output,mean(iqr_y),rho_iqr,std(iqr_y),cor_output_iqr,mean(ue_q),mean(eu_q),mean(ee_q),e_UE_theta,mean(avgfrmsize),mean(entry_q./hi_q)]);

moments=[rho_output,std_output,mean(iqr_y),rho_iqr,std(iqr_y),cor_output_iqr,mean(ue_q),mean(eu_q),mean(ee_q),e_UE_theta,mean(avgfrmsize),mean(entry_q./hi_q)];
tolerance=[1e-2,5e-3,2e-2,2e-2,4e-2,5e-2,2e-2,2e-3,2e-3,2e-2,5e-1,2e-2];

if ~isempty(find(isnan(moments) | imag(moments)~=0))
    error=1;
end




