% ============================================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% counterfact (param,vars,res,ubound_y,lbound_y,ubound_s,lbound_s): 
% produces counterfactual time-series for labor market variables.
%
% Usage: - 'param' is a structure of parameters for the model
% - 'vars' is a structure of variables created by 'initialize_vars(param)'
% - 'res' is a structure of results for the model from 'solve(param,vars)'
% - 'ubound_*' and others are arrays of size (T,2) where T is the number of quarters
% for the period studied in the case of y and the number of years in the
% case of s. These correspond to the bounds in the binary search algorithm
% in the case with y+s and y only. Can be passed as argument in order to
% avoid recalculating them every time.
%
% ============================================================================================


function [ubound_y,lbound_y,ubound_s,lbound_s]=counterfact(param,vars,res,ubound_y,lbound_y,ubound_s,lbound_s)

close all;
DETREND='Qfreq';
DISPERSION_SERIES = 'IQR'; % STD or IQR
STEP_CV=1/5;

NBER_recession_start = [1973.75,1980,1990.5,2001,2007.75];
NBER_recession_end = [1975,1980.5,1991,2001.75,2009.25];
NBER_recession_label = {'1973Q4-1975Q1','1980Q1-1982Q4','1990Q3-1991Q1','2001Q1-2001Q4','2007Q4-2009Q2'};
% note I collapse the two recessions 1980-1980.5 and 1981.5-1982.75 to
% the same because it is arguably the same recession in terms of
% unemployment

start_date=1972; % cover period from 1972Q1-2009Q4
end_date=2009; %included
end_date_q=2009.75;
end_date_m=2009+11/12;


% -------------------------------------
% Load and format empirical data

load data/MISC_DATA
load data/BED_DATA
load data/JOLTS_DATA
load data/BLOOM_DATA
load data/FRED_DATA
load data/CONS_DATA
load data/PRESCOTT_DATA

start_date=1972; % load data before 2008 to compute trend
BED_start_date=1992.5;
BED_end_date=2009.75;
JOLTS_start_date=2001;
JOLTS_end_date=2009+11/12;

d_output_q = d_output((start_date-1947)*4+1:(end_date_q-1947)*4+1); %quarterly
d_outpp_q = d_outpp((start_date-1947)*4+1:(end_date_q-1947)*4+1); %quarterly
d_unemp = d_unemp(int32((start_date-1948)*12+1:(end_date_m-1948)*12+1)); %monthly
d_labforce = d_labforce(int32((start_date-1948)*12+1:(end_date_m-1948)*12+1)); %monthly
d_entry_q((BED_start_date-start_date)*4+1:(BED_end_date-start_date)*4+1,1) = BED_DATA2((BED_start_date-1992.5)*4+1:(BED_end_date-1992.5)*4+1,1); %quarterly
d_exit_q((BED_start_date-start_date)*4+1:(BED_end_date-start_date)*4+1,1) = BED_DATA2((BED_start_date-1992.5)*4+1:(BED_end_date-1992.5)*4+1,2); %quarterly
d_hi(int32((JOLTS_start_date-start_date)*12+1):int32((JOLTS_end_date-start_date)*12+1),1) = JOLTS_DATA(int32((JOLTS_start_date-2001)*12+2):int32((JOLTS_end_date-2001)*12+2),1); %monthly from Jan01 to Mar10
d_lf(int32((JOLTS_start_date-start_date)*12+1):int32((JOLTS_end_date-start_date)*12+1),1) = JOLTS_DATA(int32((JOLTS_start_date-2001)*12+2):int32((JOLTS_end_date-2001)*12+2),5); %monthly
d_qu(int32((JOLTS_start_date-start_date)*12+1):int32((JOLTS_end_date-start_date)*12+1),1) = JOLTS_DATA(int32((JOLTS_start_date-2001)*12+2):int32((JOLTS_end_date-2001)*12+2),7); %monthly
d_sp(int32((JOLTS_start_date-start_date)*12+1):int32((JOLTS_end_date-start_date)*12+1),1) = JOLTS_DATA(int32((JOLTS_start_date-2001)*12+2):int32((JOLTS_end_date-2001)*12+2),9); %monthly
d_v(int32((JOLTS_start_date-start_date)*12+1):int32((JOLTS_end_date-start_date)*12+1),1) = JOLTS_DATA(int32((JOLTS_start_date-2001)*12+2):int32((JOLTS_end_date-2001)*12+2),3); %monthly
d_iqr_y = BLOOM_DATA(start_date-1972+1:end_date-1972+1,11); %yearly 2y
d_std_y = BLOOM_DATA(start_date-1972+1:end_date-1972+1,8); %yearly 2y

d_iqr_y = BLOOM_DATA(start_date-1972+1:end_date-1972+1,5); %yearly 25y
d_std_y = BLOOM_DATA(start_date-1972+1:end_date-1972+1,2); %yearly 25y
d_cons_q = CONS_DATA((start_date-1947)*4+1:(end_date_q-1947)*4+1,2); %quarterly consumption expenditure data
d_labor_q = PRESCOTT_DATA((start_date-1947.5)*4+1:(end_date_q-1947.5)*4+1,2);


% Create some quarterly series out of the monthly ones
d_labforce_q = averageM2Q(d_labforce);
d_unemp_q = averageM2Q(d_unemp);
d_hi_q = averageM2Q(d_hi)./d_labforce_q;
d_sp_q = averageM2Q(d_sp)./d_labforce_q;
d_qu_q = averageM2Q(d_qu)./d_labforce_q;
d_lf_q = averageM2Q(d_lf)./d_labforce_q;
d_v_q = averageM2Q(d_v)./d_labforce_q;
d_entry_q = d_entry_q ./ d_labforce_q;
d_exit_q = d_exit_q ./ d_labforce_q;

% Compute log deviations

d_logoutput_dev = logdev(d_output_q,DETREND);

d_logoutpp_dev = logdev(d_outpp,DETREND); % remove trend up until 2013 to better end-of-sample properties
d_logoutpp_dev = d_logoutpp_dev((start_date-1947)*4+1:(end_date_q-1947)*4+1);
d_logunemp_dev = logdev(d_unemp_q,DETREND);

d_loghi_dev=NaN*ones(length(start_date:.25:end_date_q),1);
d_loglf_dev=NaN*ones(length(start_date:.25:end_date_q),1);
d_logsp_dev=NaN*ones(length(start_date:.25:end_date_q),1);
d_logqu_dev=NaN*ones(length(start_date:.25:end_date_q),1);
d_logv_dev=NaN*ones(length(start_date:.25:end_date_q),1);
d_logentry_dev=NaN*ones(length(start_date:.25:end_date_q),1);
d_logexit_dev=NaN*ones(length(start_date:.25:end_date_q),1);


d_logentry_dev((BED_start_date-start_date)*4+1:(BED_end_date-start_date)*4+1) = logdev(d_entry_q((BED_start_date-start_date)*4+1:(BED_end_date-start_date)*4+1),DETREND);
d_logexit_dev((BED_start_date-start_date)*4+1:(BED_end_date-start_date)*4+1) = logdev(d_exit_q((BED_start_date-start_date)*4+1:(BED_end_date-start_date)*4+1),DETREND);
d_loghi_dev((JOLTS_start_date-start_date)*4+1:(end_date_q-start_date)*4+1) = logdev(d_hi_q((JOLTS_start_date-start_date)*4+1:(end_date_q-start_date)*4+1),DETREND);
d_logsp_dev((JOLTS_start_date-start_date)*4+1:(end_date_q-start_date)*4+1) = logdev(d_sp_q((JOLTS_start_date-start_date)*4+1:(end_date_q-start_date)*4+1),DETREND);
d_logqu_dev((JOLTS_start_date-start_date)*4+1:(end_date_q-start_date)*4+1) = logdev(d_qu_q((JOLTS_start_date-start_date)*4+1:(end_date_q-start_date)*4+1),DETREND);
d_loglf_dev((JOLTS_start_date-start_date)*4+1:(end_date_q-start_date)*4+1) = logdev(d_lf_q((JOLTS_start_date-start_date)*4+1:(end_date_q-start_date)*4+1),DETREND);
d_logv_dev((JOLTS_start_date-start_date)*4+1:(end_date_q-start_date)*4+1) = logdev(d_v_q((JOLTS_start_date-start_date)*4+1:(end_date_q-start_date)*4+1),DETREND);
d_logiqr_dev = logdev(d_iqr_y,'Yfreq');
d_logstd_dev = logdev(d_std_y,'Yfreq');


% -------------------------
% Generate counterfactuals


T=end_date-start_date+1; % number of years
lag=2; % number of lagged years
Y0=ceil(param.ny/2);
S0=ceil(param.ns/2);


% compute reference economy for log deviations
yref=vars.y_grid(Y0);
sref=vars.s_grid(S0);
[uref,gref]=findss_value(yref,sref,param,vars,res);
[iqrref,stdref]=computeIQRSTD(param,vars,res,sref,gref);
[~,~,outppref,vref,thetaref,ueref,euref,eeref,entryref,exitref,hiref,spref,quref,lfref,outputref,error] = distdyn_value(uref,gref,yref,sref,sref,param,vars,res);

% start looking for the best fit by binary search algorithm

if nargin<5    
    ubound_s=vars.s_grid(param.ns)*ones(T,2); % upper bound on series of annual shocks to s
    lbound_s=vars.s_grid(1)*ones(T,2); % lower bound on series of annual shocks to s
    
    ubound_y=vars.y_grid(param.ny)*ones(4*T,2); % upper bound on series of quarterly shocks to y
    lbound_y=vars.y_grid(1)*ones(4*T,2); % lower bound on series of quarterly shocks to y
end


% -----------
% Init search
%

success = false;
best_score_s = inf;
best_score_y = inf;
best_score_y_only = inf;
has_s_converged = false;
has_y_converged = false;
has_y_only_converged = false;
has_changed_variable = false;
fail = false;
current_phase = 1;
counter=0;

fprintf('Beginning convergence on s.\n');

while ~(success || fail) % FIRST PHASE: calibrating s and y series to match time series
    
    % evaluate current series in the middle of the bounds
    current_y=.5*(ubound_y(:,current_phase)+lbound_y(:,current_phase));
    current_s=exp(.5*(log(ubound_s(:,current_phase))+log(lbound_s(:,current_phase))));
    
    % interpolate the series of shocks to get monthly series    
    s=createIQRSeries(current_s,T,lag);
    y=createYSeries(current_y,T,lag);
    
    
    fprintf('Iteration No.%d ',counter);
    
    [tmpunemp,tmpiqr_y,tmpstd_y,tmpoutpp,tmpv,tmptheta,tmpue,tmpeu,tmpee,tmpentry,tmpexit,tmphi,tmpsp,tmpqu,tmplf,tmpoutput,error,tmpavgfrmsize,tmpprody,tmpmassfirm,tmpcons] = simulate(y,s,param,vars,res,uref,gref);
    
    
    % ------------------------------------
    % Format simulated data
    
    % Get rid of the first months
    
    unemp(:,current_phase)=tmpunemp(lag*12+1:(lag+T)*12);
    outpp(:,current_phase)=tmpoutpp(lag*12+1:(lag+T)*12);
    output(:,current_phase)=tmpoutput(lag*12+1:(lag+T)*12);
    v(:,current_phase)=tmpv(lag*12+1:(lag+T)*12);
    theta(:,current_phase)=tmptheta(lag*12+1:(lag+T)*12);
    entry(:,current_phase)=tmpentry(lag*12+1:(lag+T)*12);
    exit(:,current_phase)=tmpexit(lag*12+1:(lag+T)*12);
    hi(:,current_phase)=tmphi(lag*12+1:(lag+T)*12);
    sp(:,current_phase)=tmpsp(lag*12+1:(lag+T)*12);
    qu(:,current_phase)=tmpqu(lag*12+1:(lag+T)*12);
    lf(:,current_phase)=tmplf(lag*12+1:(lag+T)*12);
    cons(:,current_phase)=tmpcons(lag*12+1:(lag+T)*12);
    labor(:,current_phase)=1-unemp(:,current_phase);
    
    % Quarterly averages of simulated data
    
    y_q(:,current_phase) = averageM2Q(y);
    s_q(:,current_phase) = averageM2Q(s);
    output_q(:,current_phase) = averageM2Q(output(:,current_phase));
    outpp_q(:,current_phase) = averageM2Q(outpp(:,current_phase));
    unemp_q(:,current_phase) = averageM2Q(unemp(:,current_phase));
    entry_q(:,current_phase) = 3*averageM2Q(entry(:,current_phase));
    exit_q(:,current_phase) = averageM2Q(exit(:,current_phase));
    hi_q(:,current_phase) = averageM2Q(hi(:,current_phase));
    sp_q(:,current_phase) = averageM2Q(sp(:,current_phase));
    qu_q(:,current_phase) = averageM2Q(qu(:,current_phase));
    lf_q(:,current_phase) = averageM2Q(lf(:,current_phase));
    v_q(:,current_phase) = averageM2Q(v(:,current_phase));
    theta_q(:,current_phase) = averageM2Q(theta(:,current_phase));
    cons_q(:,current_phase) = averageM2Q(cons(:,current_phase));
    labor_q(:,current_phase) = averageM2Q(labor(:,current_phase));
           
    
    % Compute log deviations
    
    
    logy_dev(:,current_phase) = log(y_q(:,current_phase)) - log(yref);
    logs_dev(:,current_phase) = log(s_q(:,current_phase)) - log(sref);
    logoutput_dev(:,current_phase) = log(output_q(:,current_phase)) - log(outputref);
    logoutpp_dev(:,current_phase) = log(outpp_q(:,current_phase)) - log(outppref);
    logunemp_dev(:,current_phase) = log(unemp_q(:,current_phase)) - log(uref);
    logentry_dev(:,current_phase) = log(entry_q(:,current_phase)) - log(entryref);
    logentry_dev(entry_q(:,current_phase)<0,current_phase)=NaN;
    logexit_dev(:,current_phase) = log(exit_q(:,current_phase)) - log(exitref);
    loghi_dev(:,current_phase) = log(hi_q(:,current_phase)) - log(hiref);
    logsp_dev(:,current_phase) = log(sp_q(:,current_phase)) - log(spref);
    logqu_dev(:,current_phase) = log(qu_q(:,current_phase)) - log(quref);
    loglf_dev(:,current_phase) = log(lf_q(:,current_phase)) - log(lfref);
    logv_dev(:,current_phase) = log(v_q(:,current_phase)) - log(vref);
    logtheta_dev(:,current_phase) = log(theta_q(:,current_phase)) - log(thetaref);
    
    % Retrieve IQR/STD series and format
    
    iqr_y(:,current_phase)=tmpiqr_y(lag-1+1:lag-1+T);
    logiqr_dev(:,current_phase)=log(iqr_y(:,current_phase))-log(iqrref);
    
    std_y(:,current_phase)=tmpstd_y(lag-1+1:lag-1+T);
    logstd_dev(:,current_phase)=log(std_y(:,current_phase))-log(stdref);
    
    switch DISPERSION_SERIES
        case 'STD'
            logdisp_dev(:,current_phase)=logstd_dev(:,current_phase);
            d_logdisp_dev=d_logstd_dev;
        case 'IQR'
            logdisp_dev(:,current_phase)=logiqr_dev(:,current_phase);
            d_logdisp_dev=d_logiqr_dev;
        otherwise
            fprintf('%s.m: unknown dispersion series to use.\n',mfilename);
    end
    
    % -----------------------------------
    % Evaluate the fit with outpp and std
    %
    
    CV_tol=1e-2;
    
    if current_phase==1
        %test for s
        score_s=sqrt(sum((logdisp_dev(:,current_phase)-d_logdisp_dev).^2));
        if has_s_converged == false
            fprintf('Score_s = %d.\n',score_s);
        end
        
        if score_s<best_score_s
            best_score_s=score_s;
            best_s = current_s;
        end
        
        if best_score_s<CV_tol || (counter>=100 && best_score_s<5e-1)
            current_s = best_s;
            score_s = best_score_s;
            if has_s_converged ==false
                fprintf('\nConvergence on s reached in %d iterations with fit score_s=%2.3f.\n',counter,best_score_s);
                fprintf('Beginning convergence on y.\n');
                has_changed_variable=true;
            end
            has_s_converged = true;
        elseif counter>=100 && best_score_s>=1e-1
            has_s_converged = false;
            fail=true;
        end
        
        %test for y
        score_y=sqrt(sum((logoutpp_dev(:,current_phase)-d_logoutpp_dev).^2));
        if has_y_converged == false && has_s_converged==true && has_changed_variable==false
            fprintf('Score_y = %d.\n',score_y);
        end
        has_changed_variable=false;
        
        if score_y<best_score_y
            best_score_y=score_y;
            best_y = current_y;
        end
        
        if best_score_y<CV_tol || (counter>=200 && best_score_y<1e-1)
            current_y = best_y;
            score_y = best_score_y;
            has_y_converged = true;
        elseif counter>=200 && best_score_y>=1e-1
            has_y_converged = false;
            fail=true;
        end
        
        if has_s_converged && has_y_converged
            fprintf('\nConvergence on y reached in %d iterations with fit score_y=%2.3f.\n',counter,best_score_y);
            fprintf('Beginning convergence on y only.\n');
            
            current_phase=2;
            
        end
        
        % update bounds
        if has_s_converged==false
            for j=1:T
                if logdisp_dev(j,current_phase)>d_logdisp_dev(j)
                    ubound_s(j,current_phase)=(1-STEP_CV)*ubound_s(j,current_phase)+STEP_CV*current_s(j);
                end
                if logdisp_dev(j,current_phase)<d_logdisp_dev(j)
                    lbound_s(j,current_phase)=(1-STEP_CV)*lbound_s(j,current_phase)+STEP_CV*current_s(j);
                end
            end
        elseif has_y_converged==false
            for j=1:4*T
                if logoutpp_dev(j,current_phase)>d_logoutpp_dev(j)
                    ubound_y(j,current_phase)=(1-STEP_CV)*ubound_y(j,current_phase)+STEP_CV*current_y(j);
                end
                if logoutpp_dev(j,current_phase)<d_logoutpp_dev(j)
                    lbound_y(j,current_phase)=(1-STEP_CV)*lbound_y(j,current_phase)+STEP_CV*current_y(j);
                end
            end
        end
    elseif current_phase==2
            
        %test for y
        score_y=sqrt(sum((logoutpp_dev(:,current_phase)-d_logoutpp_dev).^2));
        if has_y_only_converged == false
            fprintf('Score_y = %d.\n',score_y);
        end       
        
        if score_y<best_score_y_only
            best_score_y_only=score_y;
            best_y_only = current_y;
            best_s_y_only = current_s;
        end
        
        if best_score_y_only<CV_tol || (counter>=300 && best_score_y_only<1e-1)
            current_y = best_y_only;
            score_y_only = best_score_y_only;
            has_y_only_converged = true;
        elseif counter>=300 && best_score_y_only>=1e-1
            has_y_only_converged = false;
            fail=true;
        end
        
        if has_s_converged && has_y_converged && has_y_only_converged
            success=true;
            fail=false;
        end
        
        if success
            fprintf('\nConvergence reached in %d iterations with fit score_s=%2.3f, score_y=%2.3f, score_y_only=%2.3f.\n',counter,best_score_s,best_score_y,best_score_y_only);
        end
        
        % update bounds
        
        if has_y_only_converged==false
            for j=1:4*T
                if logoutpp_dev(j,current_phase)>d_logoutpp_dev(j)
                    ubound_y(j,current_phase)=(1-STEP_CV)*ubound_y(j,current_phase)+STEP_CV*current_y(j);
                end
                if logoutpp_dev(j,current_phase)<d_logoutpp_dev(j)
                    lbound_y(j,current_phase)=(1-STEP_CV)*lbound_y(j,current_phase)+STEP_CV*current_y(j);
                end
            end
        end
    else
        fprintf('%s.m: error unknown phase.\n',mfilename);
    end
    
    
    counter=counter+1;
    
end

% --------------------------------------------------------
% Now, identify the peaks and troughs in various variables
%

% First, plot fitted series and imputed shock series

figure('Units','inches','Position',[0,0,7.5,5]);
set(gcf,'PaperUnits',get(gcf,'Units'),'PaperPosition',get(gcf,'Position')+[-.25 2.5 0 0]);    
set(gcf,'Name','Fitted series and imputed shocks');

margin = 0.05;
legend_w = 1-4*margin;
legend_h = .1;
axes_w_percent = .75;
axes_h_percent = .75;
plots_per_row=2;
nbrows=2;

i=1;
x=mod(i-1,plots_per_row);
y=floor((i-1)/plots_per_row);
subplot('Position',[margin+(1-2*margin)/plots_per_row*(x+(1-axes_w_percent)/2) margin+legend_h+(1-2*margin-legend_h)/nbrows*(nbrows-1-y+(1-axes_h_percent)/2) axes_w_percent*(1-2*margin)/plots_per_row axes_h_percent*(1-2*margin-legend_h)/nbrows]);
hold on;
% plot(1972:.25:2009.75,logdev(d_outpp_q,'Q'),'Color',[.5 0 .25]);
plot(1972:.25:2009.75,logdev(d_outpp_q,'Qfreq'),'k','Linewidth',1);
dashline(1972:.25:2009.75,logdev(d_outpp_q,'Q'),.5,.5,.5,.5,'Color',[0 0 0]);
plot(1972:.25:2009.75,logoutpp_dev(:,1),'Color',[0 .25 .75]);
plot(1972:.25:2009.75,logoutpp_dev(:,2),'--','Color',[0 .5 .25]);
set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
title('(a) Output-per-person','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times','interpreter','latex');
grid on;
box on;
hold off;

ld=legend('Data (band-pass)','Data (HP)','Model (y+v)','Model (y only)',-1,'Orientation','horizontal'); % set the legend out of plot
set(ld,'Position',[2*margin margin legend_w legend_h],'Box','off');
lines=findobj(ld,'Type','Line');
% set(lines(6),'LineStyle',':');

i=2;
x=mod(i-1,plots_per_row);
y=floor((i-1)/plots_per_row);
subplot('Position',[margin+(1-2*margin)/plots_per_row*(x+(1-axes_w_percent)/2) margin+legend_h+(1-2*margin-legend_h)/nbrows*(nbrows-1-y+(1-axes_h_percent)/2) axes_w_percent*(1-2*margin)/plots_per_row axes_h_percent*(1-2*margin-legend_h)/nbrows]);
hold on;
plot(1972:2009,logdev(d_iqr_y,'Yfreq'),'k','Linewidth',1);
dashline(1972:2009,logdev(d_iqr_y,'Y'),.5,.5,.5,.5,'Color',[0 0 0]);
plot(1972:2009,logiqr_dev(:,1),'Color',[0 .25 .75]);
plot(1972:2009,logiqr_dev(:,2),'--','Color',[0 .5 .25]);
ylim([-.1 .2]);
set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
title('(b) IQR','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times','interpreter','latex');
grid on;
box on;
hold off;

i=3;
x=mod(i-1,plots_per_row);
y=floor((i-1)/plots_per_row);
subplot('Position',[margin+(1-2*margin)/plots_per_row*(x+(1-axes_w_percent)/2) margin+legend_h+(1-2*margin-legend_h)/nbrows*(nbrows-1-y+(1-axes_h_percent)/2) axes_w_percent*(1-2*margin)/plots_per_row axes_h_percent*(1-2*margin-legend_h)/nbrows]);
hold on;
plot(1972:.25:2009.75,y_q(9:end,1),'Color',[0 .25 .75]);
plot(1972:.25:2009.75,y_q(9:end,2),'--','Color',[0 .5 .25]);
set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
title('(c) Imputed $y$ series','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times','interpreter','latex');
grid on;
box on;
hold off;

i=4;
x=mod(i-1,plots_per_row);
y=floor((i-1)/plots_per_row);
subplot('Position',[margin+(1-2*margin)/plots_per_row*(x+(1-axes_w_percent)/2) margin+legend_h+(1-2*margin-legend_h)/nbrows*(nbrows-1-y+(1-axes_h_percent)/2) axes_w_percent*(1-2*margin)/plots_per_row axes_h_percent*(1-2*margin-legend_h)/nbrows]);
hold on;
plot(1972:.25:2009.75,s_q(9:end,1),'Color',[0 .25 .75]);
plot(1972:.25:2009.75,s_q(9:end,2),'--','Color',[0 .5 .25]);
set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
title('(d) Imputed $v$ series','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times','interpreter','latex');
grid on;
box on;
hold off;



print('-depsc2','graphs/shocks.eps');



% Plot output and unemployment for all the recessions
plot_variable(1972:.25:2009.75,[d_logunemp_dev,logunemp_dev],-1,NBER_recession_start,NBER_recession_end,NBER_recession_label,'Unemployment','graphs/counterfact_unemp.eps');
plot_variable(1972:.25:2009.75,[d_logoutput_dev,logoutput_dev],+1,NBER_recession_start,NBER_recession_end,NBER_recession_label,'Output','graphs/counterfact_output.eps');

% Format labor market variables for the last recession and plot
hi_tmp=reshape([d_loghi_dev((2001-1972)*4+1:(2009.75-1972)*4+1),loghi_dev((2001-1972)*4+1:(2009.75-1972)*4+1,:)]', [1 3 (2009.75-2001)*4+1]);
lf_tmp=reshape([d_loglf_dev((2001-1972)*4+1:(2009.75-1972)*4+1),loglf_dev((2001-1972)*4+1:(2009.75-1972)*4+1,:)]', [1 3 (2009.75-2001)*4+1]);
qu_tmp=reshape([d_logqu_dev((2001-1972)*4+1:(2009.75-1972)*4+1),logqu_dev((2001-1972)*4+1:(2009.75-1972)*4+1,:)]', [1 3 (2009.75-2001)*4+1]);
v_tmp=reshape([d_logv_dev((2001-1972)*4+1:(2009.75-1972)*4+1),logv_dev((2001-1972)*4+1:(2009.75-1972)*4+1,:)]', [1 3 (2009.75-2001)*4+1]);
entry_tmp=reshape([d_entry_q((2001-1972)*4+1:(2009.75-1972)*4+1),entry_q((2001-1972)*4+1:(2009.75-1972)*4+1,:)]', [1 3 (2009.75-2001)*4+1]);
exit_tmp=reshape([d_exit_q((2001-1972)*4+1:(2009.75-1972)*4+1),exit_q((2001-1972)*4+1:(2009.75-1972)*4+1,:)]', [1 3 (2009.75-2001)*4+1]);

cyclicality=[1,-1,1,1,1,-1]; %+1 means procyclical, -1 countercyclical
% label={'Hirings','Layoffs','Quits','Vacancies','Entry','Exit'};
cyclicality=[1,-1,1,1]; %+1 means procyclical, -1 countercyclical
label={'Hirings','Layoffs','Quits','Vacancies'};

% plot_recession(2001:.25:2009.75,[hi_tmp;lf_tmp;qu_tmp;v_tmp;entry_tmp;exit_tmp],cyclicality,label,NBER_recession_start(5),NBER_recession_end(5),'Labor Market Variables 2007-2009');
plot_recession(2001:.25:2009.75,[hi_tmp;lf_tmp;qu_tmp;v_tmp],cyclicality,label,NBER_recession_start(5),NBER_recession_end(5),'Labor Market Variables 2007-2009','graphs/counterfact_labor.eps');

% Do the labor wedge exercise
% Compute the labor wedge
cons=[d_cons_q,cons_q];
labor=[d_labor_q,labor_q];
output=[d_output_q,output_q];

nu=.25;
wedge=cons.*labor.^(1+nu)./output;
logwedge_dev=zeros(size(wedge));
WEDGE_DETREND='Qfreq';
for j=1:3
    logwedge_dev(:,j)=logdev(wedge(:,j),WEDGE_DETREND);
    logcons_dev(:,j)=logdev(cons(:,j),WEDGE_DETREND);
    loglabor_dev(:,j)=logdev(labor(:,j),WEDGE_DETREND);
end

plot_variable(1972:.25:2009.75,logwedge_dev,+1,NBER_recession_start,NBER_recession_end,NBER_recession_label,'Labor Wedge','graphs/counterfact_wedge.eps');


end


% ------------------- %
% Auxiliary functions %
% ------------------- %


% This function plots different versions of a series for the last NBER recessions
% centering on the peak before the recession. Cyclicality is a variable +1
% if procyclical (centered on peaks) or -1 if countercylical (centered on troughs)
function plot_variable(t,series,cyclicality,NBER_recession_start,NBER_recession_end,NBER_recession_label,series_title,file)

[T,nseries]=size(series); 
N=length(NBER_recession_start); % N identifies the recessionary episode

ref=zeros(N,nseries); 
ref_id=zeros(N,nseries);

nb_styles = 3;
linestyle = {'-','-.','--'};
linewidth = {1,1,1};
colorstyle = {[0 0 0],[0 .25 .75],[0 .5 .25]};
% colorstyle = {[0 0 0],[0 0 0],[0 0 0]};

figure('Units','inches','Position',[0,0,7.5,4]);
set(gcf,'PaperUnits',get(gcf,'Units'),'PaperPosition',get(gcf,'Position')+[-.25 2.5 0 0]);    
set(gcf,'Name',series_title);

if N>3
    col=3;
    row=ceil(N/3);
else
    col=N;
    row=1;
end

for i=1:N
    for j=1:nseries
        [ref(i,j),ref_id(i,j)]=find_peak(t,cyclicality*series(:,j),NBER_recession_start(i),NBER_recession_end(i));
    end
    ref(:,3) = ref(:,2); % force peaks to coincide for the two countefactual series
    ref_id(:,3) = ref_id(:,2);
    
    subplot(row,col,i);
    hold on;
    title(sprintf('(%c) %s',97+i-1,char(NBER_recession_label(i))),'FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
%     nb_periods=max(find(max(repmat(ref(i,:),[16 1])+(0:15)'*0.25*ones(1,nseries),[],2)<=t(end)));
    series_nb_periods=min(T-ref_id(i,:)+1,24);
    nb_periods=max(series_nb_periods,[],2);
    series_plot=NaN*ones(nb_periods,nseries);    
    for j=1:nseries          
        series_plot(1:series_nb_periods(j),j)=series(ref_id(i,j):ref_id(i,j)+series_nb_periods(j)-1,j)-series(ref_id(i,j),j);        
        style_id=1+mod(j-1,nb_styles);

        if strcmp(char(linestyle(style_id)),'-.')
            dashline(0:nb_periods-1,series_plot(:,j),.5,.5,.5,.5,'Linewidth',cell2mat(linewidth(style_id)),'Color',cell2mat(colorstyle(style_id)));
        else
            plot(0:nb_periods-1,series_plot(:,j),char(linestyle(style_id)),'Color',cell2mat(colorstyle(style_id)),'LineWidth',cell2mat(linewidth(style_id)));
        end        
    end
%     plot(0:nb_periods-1,series_plot);
    xlim([0 min(series_nb_periods)-1]);
    set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
    box on;
    grid on;
    ymin = min(series_plot(:)); 
    ymax = max(series_plot(:));
    ymin=floor(ymin*100)/100; % make sure the unit on the axis is in 1e-2, not less
    ymax=(floor(ymax*100)+1)/100;
    ylim([ymin ymax]);
        

    if(i==N)
        pos=get(gca,'Position');
        ld=legend('Data','Model (y+v)','Model (y only)',-1); % set the legend out of plot
        set(gca,'Position',pos);
        pos=get(ld,'Position');
        set(ld,'Position',pos+[0.05 0 0 0],'Box','off');
        lines=findobj(ld,'Type','Line');
%         set(lines(4),'LineStyle',':');
    end
    
    hold off;    
end


print('-depsc2',file);
end


% This function plots several variables for a given recession. Series
% is a 3 dimensional array indexed by (N,nseries,T): N is the label of the
% series (hi,lf,etc), nseries indexes different versions of it (data,
% simulated,etc) and T the time period. Cyclicality is an array of size
% N which indicates whether a variables is pro(+1) or coutercyclical(-1),
% label is an cell array of strings of size N with the labels of the series
function plot_recession(t,series,cyclicality,label,NBER_recession_start,NBER_recession_end,series_title,file)

[N,nseries,T]=size(series); % N references the variable

ref=zeros(N,nseries); % ref is the reference point, either 
ref_id=zeros(N,nseries);

nb_styles = 3;
linestyle = {'-','-.','--'};
linewidth = {1,1,1};
colorstyle = {[0 0 0],[0 .25 .75],[0 .5 .25]};
% colorstyle = {[0 0 0],[0 0 0],[0 0 0]};

figure('Units','inches','Position',[0,0,7.5,4]);
set(gcf,'PaperUnits',get(gcf,'Units'),'PaperPosition',get(gcf,'Position')+[-.25 2.5 0 0]);    
set(gcf,'Name',series_title);


if N>2
    plots_per_row=2;
    nbrows=ceil(N/2);
else
    plots_per_row=N;
    nbrows=1;
end

margin = 0.05;
legend_w = .2;
legend_h = .1;
axes_w_percent = .75;
axes_h_percent = .75;

for i=1:N
    for j=1:nseries
        [ref(i,j),ref_id(i,j)]=find_peak(t,cyclicality(i)*squeeze(series(i,j,:)),NBER_recession_start,NBER_recession_end);        
    end

    if i==4 
        ref(4,2)=ref(4,2)-.25; % problem of alignment for vacancies (y+v), so just realign (no change on the actual peak-trough)
        ref_id(4,2)=ref_id(4,2)-1;
    end
    
    x=mod(i-1,plots_per_row);
    y=floor((i-1)/plots_per_row);
    subplot('Position',[margin+(1-2*margin-legend_w)/plots_per_row*(x+(1-axes_w_percent)/2) margin+(1-2*margin)/nbrows*(nbrows-1-y+(1-axes_h_percent)/2) axes_w_percent*(1-2*margin-legend_w)/plots_per_row axes_h_percent*(1-2*margin)/nbrows]);
    
    hold on;
    title(sprintf('(%c) %s',97+i-1,label{i}),'FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
    nb_periods=max(find(max(repmat(ref(i,:),[24 1])+(0:23)'*0.25*ones(1,nseries),[],2)<=t(end)));
    series_plot=zeros(N,nseries,nb_periods);
    for j=1:nseries     
        series_plot(i,j,:)=series(i,j,ref_id(i,j):ref_id(i,j)+nb_periods-1)-series(i,j,ref_id(i,j));   
        style_id=1+mod(j-1,nb_styles);
        if strcmp(char(linestyle(style_id)),'-.')
            dashline(0:nb_periods-1,squeeze(series_plot(i,j,:)),.5,.5,.5,.5,'Linewidth',cell2mat(linewidth(style_id)),'Color',cell2mat(colorstyle(style_id)));
        else
            plot(0:nb_periods-1,squeeze(series_plot(i,j,:)),char(linestyle(style_id)),'Color',cell2mat(colorstyle(style_id)),'LineWidth',cell2mat(linewidth(style_id)));
        end   
        
    end

    set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
    box on;
    grid on;
    ymin = min(series_plot(i,:)); 
    ymax = max(series_plot(i,:));
    ymin=floor(ymin*10)/10; % make sure the unit on the axis is in 1e-1, not less
    ymax=(floor(ymax*10)+1)/10;
    ylim([ymin ymax]);   
    xlim([0 7]);
    hold off; 

end

ld=legend('Data','Model (y+v)','Model (y only)',-1);
set(ld,'Position',[1-margin-legend_w 1-3*margin-legend_h legend_w legend_h],'Box','off');
lines=findobj(ld,'Type','Line');
% set(lines(4),'LineStyle',':');

print('-depsc2',file);
ref
end



% This function identifies the peak date preceding a recession starting
% at start_date in the the 'series' corresponding to dates 't'. A peak is
% defined as the first local maximum encountered before the trough during the
% recession period that follows a period of at least 3 periods of growth (to avoid tiny blips to be wrongly identified as peaks.)
% NB: to find troughs, just multiply the series by -1.
function [peak,peak_id] = find_peak(t,series,start_date,end_date)

start_date_id = max(find(t<=start_date));
end_date_id = max(find(t<=end_date));
[trough,trough_id]=min(series(start_date_id:min(end_date_id,length(t)))); % look for the min over the recession 
cursor=start_date_id-1 + trough_id; 
maximum = -inf;
found = false;
counter = 0;
if trough_id>1
    min_found=true;
else 
    min_found=false;
end
while ~found && cursor >=1
    if min_found==false
        if series(cursor) <= trough
            trough = series(cursor);
        else
            min_found=true;
        end
    else
        if series(cursor) < series(cursor+1)
%               if series(cursor) < maximum % (old method)
            if counter==0
                peak_id=cursor+1;
            end
            counter=counter+1;
        else
            counter=0;
            maximum = series(cursor);
        end
        if counter==2
            found=true;
        end
    end
    cursor=cursor-1;
end
if ~found && cursor==0 % if reach beginning of sample, set the trough to beginning
    peak_id = 1;
end
peak=t(peak_id);

end

% This function interpolates a yearly series of s linearly to a monthly
% series with coincidence in the sixth month (june). Initial and final
% conditions are set to the first and last values in the series.
function s = createIQRSeries(current_s,T,lag)
s=zeros((T+lag)*12,1);

% interpolate the series of shocks to get monthly series
s(1:lag*12+6)=current_s(1);
s((lag+T)*12-5:(lag+T)*12)=current_s(T);

for j=1:T-1
    s((lag+j-1)*12+(7:18))=current_s(j+1)*(1:12)/12+current_s(j)*(11:-1:0)/12;
end

end

% This function interpolates a quarterly series of y linearly to a monthly
% series with coincidence in the 2 month of the quarter. Initial and final
% conditions are set to the first and last values in the series.
function y = createYSeries(current_y,T,lag)
y=zeros((T+lag)*12,1);

% interpolate the quarterly series of shocks to get monthly series
y(1:lag*12+2)=current_y(1);
y((lag+T)*12-1:(lag+T)*12)=current_y(4*T);

for j=1:4*T-1
    y(lag*12+(j-1)*3+(2:4))=current_y(j+1)*(0:2)/3+current_y(j)*(3:-1:1)/3;
end

end


% This function computes reference IQR for a given stationary value of s,
% useful to compute log deviations
function [mean_iqr,mean_std] = computeIQRSTD(param,vars,res,s,gref)
% Draw a population of firms to compute IQR
rng(0,'twister'); % reset the random number generator seed, so that the function
% produces consistent results
N=10000;
T=30; % number of years simulated
z=zeros(N,T);

[firm_z0,~]=draw_firms(gref,N,param); % draw a population of N firms from the steady-state distribution
firm_z0=vars.z_grid(firm_z0);

for t=1:(T-1)*12+1
    firm_z1=draw_z_value(s,firm_z0,param);
    
    if(mod(t-1,12)==0) % plot progress every year
        z(:,floor((t-1)/12)+1)=firm_z1;
    end
    
    firm_z0=firm_z1;
end

% regress z_t on z_{t-1}, year fixed effect and firm fixed effect
A=z(1:N*(T-1))';
B=z(N+1:N*T)';
[rho,stdrho,mse]=lscov(A,B);
e_hat=z(:,2:T)-rho*z(:,1:T-1);

iqr_y=iqr(e_hat)';
mean_iqr=mean(iqr_y);
std_y=std(e_hat)';
mean_std=mean(std_y);

end

