% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% distdyn(u0,g0,y1,s1,s0,param,vars,res): computes the one period evolution
% of the employment across firms distribution g and other variables
%
% Instructions: - 'u0' unemployment rate at the beginning of the period
% - 'g0' employment distribution over (z,n) at beginning of period
% - 'y1,s1' shocks to y and s in the current period (as grid indices)
% - 's0' previous period s shock (applies currently given timing)
% - 'param' is a structure of parameters for the model
% - 'vars' is a structure of variables created by 'initialize_vars(param)'
% - 'res' is a structure of results for the model from 'solve(param,vars)'
%
% ========================================================================

function [u1,g1,outpp,v,theta,UE,EU,EE,entry,exit,hi,sp,qu,lf,output,err,cons] = distdyn(u0,g0,y1,s1,s0,param,vars,res)
    
    err=false; %error flag to signal negative entry
    
    % Initialization
    %
    
    firm_entry=0;
    
    
    % Extract the following useful objects from optimal policies
    [z_id,n_id]=ndgrid(1:param.nz,1:param.nn);
    Gn=reshape(res.gn(y1,s1,:,:),param.nz*param.nn,1);
    [~,gn_id]=histc(Gn,vars.n_grid);
    Ibound=find(gn_id==param.nn);
    Iwithin=find(gn_id<param.nn);
    Gnhat = zeros(param.nz*param.nn,param.nz*param.nn);
    Gnhat(z_id(Iwithin)+(gn_id(Iwithin)-1)*param.nz+(Iwithin-1)*param.nz*param.nn)=(vars.n_grid(gn_id(Iwithin)+1)-Gn(Iwithin))./(vars.n_grid(gn_id(Iwithin)+1)-vars.n_grid(gn_id(Iwithin)));
    Gnhat(z_id(Iwithin)+(gn_id(Iwithin)+1-1)*param.nz+(Iwithin-1)*param.nz*param.nn)=(Gn(Iwithin)-vars.n_grid(gn_id(Iwithin)))./(vars.n_grid(gn_id(Iwithin)+1)-vars.n_grid(gn_id(Iwithin)));
    Gnhat(z_id(Ibound)+(gn_id(Ibound)-1)*param.nz+(Ibound-1)*param.nz*param.nn)=1;
    
    Ge=reshape(res.ge(y1,s1,:,:),param.nz*param.nn,1);
    Gtau=reshape(res.gtau(y1,s1,:,:),param.nz*param.nn,1);
    Gthetax=reshape(res.gtheta_x(y1,s1,:,:),param.nz*param.nn,1);
%     Gx=reshape(gx(y1,s1,:,:),param.nz*param.nn,1);
    Gne=(Gn-vars.n_grid(n_id(:))).*(Gn>=vars.n_grid(n_id(:)));
    
    
    % Compute the new firm distribution
    g1_start=reshape(squeeze(vars.pi_z(s0,:,:))'*g0,param.nz*param.nn,1); % first, apply the z shock
    g1_end=Gnhat*((1-Ge).*g1_start); % then, redistribute according to the exit and nhat shock
    
    % Now, compute the labor market flows
    
    qu=sum(param.lambda*(1-Ge).*(1-Gtau).*p(Gthetax,param).*vars.n_grid(n_id(:)).*g1_start);
    lf=sum((1-Ge).*Gtau.*vars.n_grid(n_id(:)).*g1_start);
    output=sum(exp(vars.y_grid(y1)+vars.z_grid(z_id(:))).*F(vars.n_grid(n_id(:)),param).*g1_end);
    exit=sum(Ge.*vars.n_grid(n_id(:)).*g1_start); %weight exit by employment
    firm_exit=sum(Ge.*g1_start); %exit by mass of firms
    ne_incumbent=sum(Gne.*(1-Ge).*g1_start); % only firms that survive post new vacancies
    v=res.gtheta_xu(y1,s1)*u0+sum(param.lambda*(1-Ge).*(1-Gtau).*Gthetax.*vars.n_grid(n_id(:)).*g1_start);    
    hi=u0*p(res.gtheta_xu(y1,s1),param)+qu; % compute the total numbers of hires
    lf=lf+exit;
    sp=lf+qu;
    
    u1=u0*(1-p(res.gtheta_xu(y1,s1),param))+lf; % now add the laid-off workers to the pool of unemployed
    
    % Compute entry
    %
    
    % compute the distribution of entering firms
    I=find(res.entering_firmsize_id(y1,s1,:)<2); % entering_firmsize_id is -1 if exits, vars.n_grid(1)=0 so ignore too
    firm_distribution=vars.gz; % distribution of entering firms
    firm_distribution(I)=0;
    firmsize_slice=squeeze(res.entering_firmsize_id(y1,s1,:));
    firmsize_slice(I)=2; % set 2 (!=0) instead of -1 to prevent error, but no incidence on result. don't put 1 either to prevent divide by 0
    empl_distribution=vars.n_grid(firmsize_slice).*firm_distribution;
    empl_distribution=empl_distribution/sum(empl_distribution);
    entry = hi-ne_incumbent; % number of hires by entering firms is total minus hires by incumbent
    empl_entry_mass=entry*empl_distribution;
    firm_entry_mass = empl_entry_mass./vars.n_grid(firmsize_slice); % number of new entering firms
    if entry<0        
        err=true;        
    end
    
    g1=reshape(g1_end,param.nz,param.nn);
    
    for z=1:param.nz
        if(res.entering_firmsize_id(y1,s1,z)>0)
            % update distribution with entrants
            g1(z,res.entering_firmsize_id(y1,s1,z))=g1(z,res.entering_firmsize_id(y1,s1,z))+firm_entry_mass(z); %new entrants at z with size vars.n_grid(entering_firmsize_id(y1))
            output=output+firm_entry_mass(z)*exp(vars.y_grid(y1)+vars.z_grid(z))*F(vars.n_grid(res.entering_firmsize_id(y1,s1,z)),param);
            firm_entry=firm_entry+firm_entry_mass(z)*exp(vars.y_grid(y1)+vars.z_grid(z))*F(vars.n_grid(res.entering_firmsize_id(y1,s1,z)),param);
        end
    end
    
    % Return results
    
    theta=res.gtheta_xu(y1,s1); % return theta for unemployed workers
    outpp=output/(1-u1); %return total output per person
    UE=p(res.gtheta_xu(y1,s1),param); %job finding rate of unemployed
    EU=(sp-qu)/((1-u0)*(1-UE/2)); %job separation rate following Shimer's1 method
    EE=qu/(1-u0);
    %firm_entry=sum(firm_entry_distribution);
    mass_entrants=sum(firm_entry_mass);
    mass_firm=sum(g1(:));
    cons=output-param.kf*mass_firm-param.ke*mass_entrants-param.c*v;
    
    
    
    
