% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% distdyn_value(u0,g0,y1,s1,s0,param,vars,res): computes the one period evolution
% of the employment across firms distribution g and other variables
%
% Instructions: - 'u0' unemployment rate at the beginning of the period
% - 'g0' employment distribution over (z,n) at beginning of period
% - 'y1,s1' shocks to y and s in the current period (in values)
% - 's0' previous period s shock (applies currently given timing)
% - 'param' is a structure of parameters for the model
% - 'vars' is a structure of variables created by 'initialize_vars(param)'
% - 'res' is a structure of results for the model from 'solve(param,vars)'
%
% This is a smoothed version of distdyn() which operates on grid indices
% ========================================================================
%
function [u1,g1,outpp,v,theta,UE,EU,EE,entry,exit,hi,sp,qu,lf,output,err,cons,entry_pattern] = distdyn_value(u0,g0,y1,s1,s0,param,vars,res)
    
    err=false; %error flag to signal negative entry
        
    % Extract the following useful objects from optimal policies
    [z_id,n_id]=ndgrid(1:param.nz,1:param.nn);
    
    Gn=reshape(res.gn_spline(y1*ones(param.nz,param.nn),s1*ones(param.nz,param.nn),vars.z_grid(z_id),vars.n_grid(n_id)),param.nz*param.nn,1);
    [~,gn_id]=histc(Gn,vars.n_grid);
    Ibound=find(gn_id==param.nn);
    Iwithin=find(gn_id<param.nn);
    Gnhat = zeros(param.nz*param.nn,param.nz*param.nn);
    Gnhat(z_id(Iwithin)+(gn_id(Iwithin)-1)*param.nz+(Iwithin-1)*param.nz*param.nn)=(vars.n_grid(gn_id(Iwithin)+1)-Gn(Iwithin))./(vars.n_grid(gn_id(Iwithin)+1)-vars.n_grid(gn_id(Iwithin)));
    Gnhat(z_id(Iwithin)+(gn_id(Iwithin)+1-1)*param.nz+(Iwithin-1)*param.nz*param.nn)=(Gn(Iwithin)-vars.n_grid(gn_id(Iwithin)))./(vars.n_grid(gn_id(Iwithin)+1)-vars.n_grid(gn_id(Iwithin)));
    Gnhat(z_id(Ibound)+(gn_id(Ibound)-1)*param.nz+(Ibound-1)*param.nz*param.nn)=1;
    
    Ge=res.exit_value_spline(y1*ones(param.nz*param.nn,1),s1*ones(param.nz*param.nn,1),vars.z_grid(z_id(:)),vars.n_grid(n_id(:))) > res.V_spline(y1*ones(param.nz*param.nn,1),s1*ones(param.nz*param.nn,1),vars.z_grid(z_id(:)),vars.n_grid(n_id(:)));  
    Gthetax=res.gtheta_x_spline(y1*ones(param.nz*param.nn,1),s1*ones(param.nz*param.nn,1),vars.z_grid(z_id(:)),vars.n_grid(n_id(:)));    
    Gne=(Gn-vars.n_grid(n_id(:))).*(Gn>=vars.n_grid(n_id(:)));
%         Gtau=res.gtau_spline(y1*ones(param.nz*param.nn,1),s1*ones(param.nz*param.nn,1),vars.z_grid(z_id(:)),vars.n_grid(n_id(:)));    
    Gtau=1-(Gn-Gne)./((1-param.lambda*p(Gthetax,param)).*vars.n_grid(n_id(:))); % instead of using spline, choose tau to make sure n'=n(1-tau)(1-lanbda*p)+ne. true on gridpoints, not necessarily between them
    Gtau(Gn>=vars.n_grid(n_id(:)))=0; % make sure, tau=0 for growing firms
     
    
    % Interpolate the transition matrix linearly on s
    [~,s0_id]=histc(s0,[-inf;vars.s_grid;inf]);
    s0_id=s0_id-1; % index 1 corresponds to the index in vars.s_grid
    if s0_id==1
        Gz=squeeze(vars.pi_z(1,:,:));
    elseif s0_id==param.ns
        Gz=squeeze(vars.pi_z(param.ns,:,:));
    elseif s0_id>1 && s0_id<param.ns
        Gz=(vars.s_grid(s0_id+1)-s0)/(vars.s_grid(s0_id+1)-vars.s_grid(s0_id))*squeeze(vars.pi_z(s0_id,:,:))+(s0-vars.s_grid(s0_id))/(vars.s_grid(s0_id+1)-vars.s_grid(s0_id))*squeeze(vars.pi_z(s0_id+1,:,:));
    else
        fprintf('%s.m: invalid value for s.\n',mfilename);
        err=true;
        return;
    end
    
    % Compute the new firm distribution
    g1_start=reshape(Gz'*g0,param.nz*param.nn,1); % first, apply the z shock
    g1_end=Gnhat*((1-Ge).*g1_start); % then, redistribute according to the exit and nhat shock
    
    % Now, compute the labor market flows
    
    qu=sum(param.lambda*(1-Ge).*(1-Gtau).*p(Gthetax,param).*vars.n_grid(n_id(:)).*g1_start);
    lf=sum((1-Ge).*Gtau.*vars.n_grid(n_id(:)).*g1_start);
    output=sum(exp(y1+vars.z_grid(z_id(:))).*F(vars.n_grid(n_id(:)),param).*g1_end);
    exit=sum(Ge.*vars.n_grid(n_id(:)).*g1_start); %weight exit by employment
    ne_incumbent=sum(Gne.*(1-Ge).*g1_start); % only firms that survive post new vacancies
    v=res.gtheta_xu_spline(y1,s1)*u0+sum(param.lambda*(1-Ge).*(1-Gtau).*Gthetax.*vars.n_grid(n_id(:)).*g1_start);
    hi=u0*p(res.gtheta_xu_spline(y1,s1),param)+qu; % compute the total numbers of hires
    lf=lf+exit;
    sp=lf+qu;
    
    u1=u0*(1-p(res.gtheta_xu_spline(y1,s1),param))+lf; % now add the laid-off workers to the pool of unemployed
    

    % Compute entry
    %
    
    % compute the distribution of entering firms
    entering_ge=res.exit_value_spline(y1*ones(param.nz,1),s1*ones(param.nz,1),vars.z_grid,0*ones(param.nz,1)) > res.V_spline(y1*ones(param.nz,1),s1*ones(param.nz,1),vars.z_grid,0*ones(param.nz,1));
    entering_gn=res.gn_spline(y1*ones(param.nz,1),s1*ones(param.nz,1),vars.z_grid,0*ones(param.nz,1));
    firm_distribution=vars.gz; % distribution of entering firms
    firm_distribution(entering_ge)=0;
    empl_distribution=entering_gn.*firm_distribution;
    empl_distribution=empl_distribution/sum(empl_distribution);
    entry = hi-ne_incumbent; % number of hires by entering firms is total minus hires by incumbent
    if entry<0        
        err=true;        
    end
    empl_entry_mass=entry*empl_distribution;
    firm_entry_mass = empl_entry_mass./entering_gn; % number of new entering firms
    firm_entry_mass(entering_ge==1 | entering_gn<=0)=0;
    
    
    g1=reshape(g1_end,param.nz,param.nn);
    % prepare to interpolate the size of entrants
    [~,entering_gn_id]=histc(entering_gn,vars.n_grid);
    
    for z=1:param.nz
        if(entering_ge(z)~=1)
            % update distribution with entrants
            %             g1(z,res.entering_firmsize_id(y1,s1,z))=g1(z,res.entering_firmsize_id(y1,s1,z))+firm_entry_mass(z); %new entrants at z with size vars.n_grid(entering_firmsize_id(y1))
            if entering_gn_id(z)==param.nn
                g1(z,entering_gn_id(z))=g1(z,entering_gn_id(z))+firm_entry_mass(z);
            else %interpolate
                g1(z,entering_gn_id(z)+1)=g1(z,entering_gn_id(z)+1)+(entering_gn(z)-vars.n_grid(entering_gn_id(z)))/(vars.n_grid(entering_gn_id(z)+1)-vars.n_grid(entering_gn_id(z)))*firm_entry_mass(z);
                g1(z,entering_gn_id(z))=g1(z,entering_gn_id(z))+(vars.n_grid(entering_gn_id(z)+1)-entering_gn(z))/(vars.n_grid(entering_gn_id(z)+1)-vars.n_grid(entering_gn_id(z)))*firm_entry_mass(z);
            end
            
            output=output+firm_entry_mass(z)*exp(y1+vars.z_grid(z))*F(entering_gn(z),param);
        end
    end
    
    % Return results
    
    theta=res.gtheta_xu_spline(y1,s1); % return theta for unemployed workers
    outpp=output/(1-u1); %return total output per person
    UE=p(res.gtheta_xu_spline(y1,s1),param); %job finding rate of unemployed
    EU=(sp-qu)/((1-u0)*(1-UE/2)); %job separation rate following Shimer's method
    EE=qu/(1-u0);
    mass_entrants=sum(firm_entry_mass);
    mass_firm=sum(g1(:));
    cons=output-param.kf*mass_firm-param.ke*mass_entrants-param.c*v;
    entry_pattern=g1-reshape(g1_end,param.nz,param.nn); % return the pattern of entry 
    
    
    
    
    
    
    
