% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% firm_growth_distribution (param,vars,res): plots the distribution of 
% firms growth rates
%
% Instructions: - 'param' is a structure of parameters for the model
% - 'vars' is a structure of variables created by 'initialize_vars(param)'
% - 'res' is a structure of results for the model from 'solve(param,vars)'
%
% ========================================================================


function firm_growth_distribution(param,vars,res)
Y0=1;%ceil(param.ny/2);
S0=ceil(param.ns/2);
[u0,g0]=findss(Y0,S0,param,vars,res); % find ss when y is constant equal to mean
y0=vars.y_grid(Y0);
s0=vars.s_grid(S0);

T=10; % number of quarters

% Declare all variables that we are going to keep track of
%
y=y0*ones((T+1)*3,1);
s=s0*ones((T+1)*3,1);

% draw the aggregate shocks
rng(0,'twister');
for t=2:(T+1)*3    
    [y(t),s(t)]=draw_ys_value (y(t-1),s(t-1),param,vars);        
end

% init the sample of firms
N=10000;
firm_z=zeros(N,(T+1)*3);
firm_n=zeros(N,(T+1)*3);
firm_exit=zeros(N,(T+1)*3);
[firm_z0,firm_n0]=draw_firms(g0,N,param); % draw a population of N firms from the steady-state distribution
firm_z0=vars.z_grid(firm_z0);
firm_n0=vars.n_grid(firm_n0);
firm_n1=firm_n0;

[Mz_id,Mn_id]=ndgrid(1:param.nz,2:param.nn);
    
fprintf('Simulating the economy [');


    for i=1:(T+1)*3
        y1=y(i);
        s1=s(i);
        
        % correcting for selection
      
        firm_z1=max(min(draw_z_value(s0,firm_z0,param),vars.zmax),vars.zmin);
        firm_n1=max(min(res.gn_spline(y1*ones(N,1),s1*ones(N,1),firm_z1,firm_n0),vars.nmax),vars.nmin);
        ge = res.exit_value_spline(y1*ones(N,1),s1*ones(N,1),firm_z1,firm_n0) > res.V_spline(y1*ones(N,1),s1*ones(N,1),firm_z1,firm_n0);
        firm_z(:,i)=firm_z0;
        firm_n(:,i)=firm_n0;
        I=find(ge==true); % find the exiters        
        [firm_z1_id,firm_n1_id]=draw_firms(g0,length(I),param); % replace exiters with new firms
        firm_z1(I)=vars.z_grid(firm_z1_id);
        firm_n1(I)=vars.n_grid(firm_n1_id);
        
        firm_exit(I,i)=1;
                
                       
        [u1,g1] = distdyn_value(u0,g0,y1,s1,s0,param,vars,res);
        
        
        if(floor(10*i/((T+1)*3))~=floor(10*(i-1)/((T+1)*3))) % plot progress every 10%
            fprintf('.');
        end    
        
        if i == 3*(T+1);
            fprintf(']\n');
        end
       
        
        % Update state variables 
        g0=g1;
        u0=u1;
        s0=s1;
        firm_z0=firm_z1;
        firm_n0=firm_n1;
        
    end   
    
    
    % ------------------------------------
    % Now, take care of the simulated data
    %
    
    firm_n_q=zeros(N,T+1);
    firm_z_q=zeros(N,T+1);
    firm_exit_q=zeros(N,T+1);
    
    for i=1:(T+1)
        firm_n_q(:,i)=firm_n(:,(i-1)*3+1);
        firm_z_q(:,i)=firm_z(:,(i-1)*3+1);
        firm_exit_q(:,i)=firm_exit(:,(i-1)*3+1)+firm_exit(:,(i-1)*3+2)+firm_exit(:,(i-1)*3+3);        
    end
    
    % compute growth rates
    firm_growth_q=zeros(N,T);    
    firm_gz_q=zeros(N,T);    
    for i=1:T        
        firm_growth_q(:,i)=200*(firm_n_q(:,i+1)-firm_n_q(:,i))./(firm_n_q(:,i+1)+firm_n_q(:,i));                
        firm_gz_q(:,i)=200*(firm_z_q(:,i+1)-firm_z_q(:,i))./(firm_z_q(:,i)+firm_z_q(:,i+1));                
    end
    firm_exit_q=firm_exit_q(:,1:T); % the relevant exit dummies correspond to the 3 periods between observations
    firm_n_q=0.5*(firm_n_q(:,1:T)+firm_n_q(:,2:(T+1))); % average size to weight observations
    
    firm_growth_q=firm_growth_q(:);
    firm_gz_q=firm_gz_q(:);
    firm_exit_q=firm_exit_q(:);
    firm_n_q=firm_n_q(:);
    
    % remove exiting firms
    I=find(firm_exit_q==0 & ~isnan(firm_growth_q) & firm_n_q >= 0);
    firm_growth_q=firm_growth_q(I);
    firm_gz_q=firm_gz_q(I);
    firm_n_q=firm_n_q(I);
    N=length(firm_n_q);
    
    
    
    % ------------
    % Plot results
    
    load data/DFH_DATA
    
    edges=[-inf;.5*(DFH_bin(1:end-1)+DFH_bin(2:end));inf];
    [n bin]=histc(firm_growth_q,edges);    
    count=accumarray(bin,firm_n_q,[length(edges)-1, 1]); %weight each growth rate by employment
    count=count./sum(count);
    
    close all;
    hdl=figure('Units','inches','Position',[0,0,6.5,2.5]);
    set(gcf,'PaperUnits',get(gcf,'Units'),'PaperPosition',get(gcf,'Position')+[0.75 0.25 0 0]);    
    hold on;    
    bar(DFH_lbin(21:61)+1.25,count(21:61),'FaceColor',[.65 .75 1],'EdgeColor',[.4 .4 .4],'BarWidth',.5);
    bar(DFH_lbin(21:61)+3.75,DFH_dist08(21:61),'FaceColor',[0 .25 .5],'EdgeColor',[.4 .4 .4],'BarWidth',.5);
    xlim([-100 100]);
    grid on;
    box on;
    set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
    lhdl = legend('Model','Data','FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
    set(lhdl,'Box','off');
    hold off;
    print('-depsc2','graphs/firm_growth_distribution.eps');            
    
end



