% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% impulse.m: compute and display impulse responses to y and s shocks.
%
% Instructions: simply run 'impulse()' without arguments
%
% ========================================================================

function impulse
% ---------------------
% Computing equilibrium 

% Here I recompute the baseline equilibrium with the same dimensions but
% larger grids in y and s to accomodate larger shocks... saved under
% impulse.mat.

fprintf('[-] Computing equilibrium...\n');

param = struct('A',1,'alpha',.85,'b',1.402502411917219,'beta',.996,'c',1.789395096004241,'ke',14.213281945105606,'kf',1.956256938229172,'lambda',0.366041621306308,...
    'rho_y',0.990372641907069,'sigma_y',0.042251089970073,'rho_z',(.95)^(1/3),'sigma_z',0.532628971768815,...
    'rho_s',0.98,'sigma_s',0.131668241312217,'cor_ys',-0.4,...
    'gamma',1.59929,'m',1,...
    'ny',21,'ns',15,'nz',15,'nn',50);  % these parameters correspond to calibration_full_010615 but with different grids

vars = initialize_vars(param,2,2,2,2,3,3);
Y0_id=ceil(param.ny/2);
S0_id=ceil(param.ns/2);
Y0=vars.y_grid(Y0_id);
S0=param.sigma_z;
[res,aux,error]=solve(param,vars,[],[],'GE');
% compute PE
res_PE = res;
res_PE.cost = res.cost(Y0_id + (S0_id-1)*param.ny)*ones(param.ny*param.ns,1);
[res_PE,aux_PE,error]=solve(param,vars,res_PE,[],'PE');
save('results/impulse.mat');

% load('results/impulse.mat');

% ----
% Init

T=40+1; % simulate T periods

% Init shocks

nb_series=2;
lag=5;
y=Y0*ones(nb_series,T);
s=S0*ones(nb_series,T);

y(1,lag+1)=Y0-0.01; % 1 percent shock to aggregate productivity (in logs)
s(2,lag+1)=S0*exp(0.05); % 5% shock to volatility

for t=lag+2:T
    y(:,t)=param.rho_y*y(:,t-1);
    s(:,t)=exp((1-param.rho_s)*log(param.sigma_z)+param.rho_s*log(s(:,t-1)));
end

% Init time series

output=zeros(nb_series,T);
outpp=zeros(nb_series,T);
v=zeros(nb_series,T);
unemp=zeros(nb_series,T);
hi=zeros(nb_series,T);
sp=zeros(nb_series,T);
qu=zeros(nb_series,T);
lf=zeros(nb_series,T);
entry=zeros(nb_series,T);
exit=zeros(nb_series,T);
cons=zeros(nb_series,T);

% Simulate
fprintf('[-] Computing steady state...');
[u0,g0]=findss_value(Y0,S0,param,vars,res); % find ss when y is constant equal to mean
% [u0,g0]=findss(Y0_id,S0_id,param,vars,res); % find ss when y is constant equal to mean

fprintf('\n[-] Beginning simulations...');
for i=1:nb_series
    fprintf('\n    No.%d ',i);
    [unemp(i,:),~,~,outpp(i,:),v(i,:),~,~,~,~,entry(i,:),exit(i,:),hi(i,:),sp(i,:),qu(i,:),lf(i,:),output(i,:),~,~,~,~,cons(i,:)] = simulate(y(i,:),s(i,:),param,vars,res,u0,g0);
end

nu=.25;
wedge = cons./output.*(1-unemp).^nu;


% Plot

fprintf('\n[-] Plotting results...\n');
close all;

y=exp(y);

variables = {'y','s','output','outpp','unemp','v','hi','sp','qu','lf','entry','exit'};
labels = {'y','v','output','output-per-person','unemployment','vacancies','hirings','separations','quits','layoffs','entry','exit'};
title_list = {'Productivity shock y','Volatility shock v'};
file = {'graphs/impulse_y.eps','graphs/impulse_v.eps'};
N=length(labels);

series=zeros(N,T);

for i=1:nb_series
    for j=1:N
        data=eval(char(variables(j)));
        series(j,:)=data(i,:);
    end
    
    display_series(series,char(title_list(i)),labels,char(file(i)),lag,3);
end

% Additional graph showing the response of the labor wedge

figure('Units','inches','Position',[0,0,7.5,2.5]);
set(gcf,'PaperUnits',get(gcf,'Units'),'PaperPosition',get(gcf,'Position')+[-.25 2.5 0 0]);    
set(gcf,'DefaultAxesColorOrder',[0 0 0]);
set(gcf,'Name','Labor wedge');

subplot(1,2,1);
plot(-lag:T-lag-1,log(wedge(1,:))-log(wedge(1,1)),'k');    
title('(a) -1\% productivity shock','interpreter','latex');     
set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
xlim([-lag T-lag-1]);
ylim([-0.02 0.01]);
ylabel('1-$\tau$','interpreter','latex');
grid on;
box on;
hold off;

subplot(1,2,2);
plot(-lag:T-lag-1,log(wedge(2,:))-log(wedge(2,1)),'k');    
title('(b) +5\% volatility shock','interpreter','latex');     
set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
xlim([-lag T-lag-1]);
ylim([-0.02 0.01]);
ylabel('1-$\tau$','interpreter','latex');
grid on;
box on;
hold off;

print('-depsc2','graphs/impulse_wedge.eps');



end

function display_series(series,figtitle,labels,file,lag,plots_per_row)
[N,T]=size(series);

% plots_per_row=3;
nbrows=ceil(N/plots_per_row);

series_plot=zeros(N,T);


figure('Units','inches','Position',[0,0,7.5,7.5]);
set(gcf,'PaperUnits',get(gcf,'Units'),'PaperPosition',get(gcf,'Position')+[-.25 2.5 0 0]);    
set(gcf,'DefaultAxesColorOrder',[0 0 0]);

set(gcf,'Name',figtitle);
for i=1:N
    subplot(nbrows,plots_per_row,i);
    hold on;
    series_plot(i,:)=log(series(i,:))-log(series(i,1));
    plot(-lag:T-lag-1,series_plot(i,:),'k');    
    title(sprintf('(%c) %s',97+i-1,char(labels(i))),'interpreter','latex');     
    set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
    xlim([-lag T-lag-1]);
    ymin = min(series_plot(i,:)); 
    ymax = max(series_plot(i,:));
    ymin=min(floor(ymin*100)/100,-0.01); % make sure the unit on the axis is in 1e-2, not less
    ymax=(floor(ymax*100)+1)/100;
    ylim([ymin-.05*(ymax-ymin) ymax+.05*(ymax-ymin)]);
    grid on;
    box on;
    hold off;
end

print('-depsc2',file);
end





