% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% initialize_vars (param,myl,myh,msl,msh,mzl,mzh): initialize variables 
% of the model given parameters (creates grids, transition matrices, etc.)
% all stored in the variables 'vars'.
%
% Instructions: - 'param' is a structure of parameters for the model
% - 'myl' is the number of multiple of stdev of y that corresponds to lower
% bound for the grid in y
% - 'myh' is the number of multiple of stdev of y that corresponds to upper
% bound for the grid in y
% - 'msl,msh,mzl,mzh': same but for s and z
%
% ========================================================================


function vars=initialize_vars(param,myl,myh,msl,msh,mzl,mzh)
    vars = struct('y_grid',[],'s_grid',[],'z_grid',[],'n_grid',[],...
        'ymax',[],'ymin',[],'zmax',[],'zmin',[],'gz',[],...
        'smax',[],'smin',[],'pi_ys',[],'pi_ysz',[],'pi_z',[],...
        'alpha_F',[],'alpha_V',[]);
    
    
    % Create grid for y
    %
    if ~exist('myl')        
        myl = 4; % range of grid y - lower bound in standard deviations
    end
    if ~exist('myh')        
        myh = 4; % upper bound
    end
    
    vars.y_grid = linspace(-myl*param.sigma_y,myh*param.sigma_y,param.ny)';
    if param.ny==1
        vars.y_grid=(-myl*param.sigma_y+myh*param.sigma_y)/2;
    end
    vars.ymax = max(vars.y_grid);
    vars.ymin = min(vars.y_grid);
    
        
    % Create grid for s
    %
    if ~exist('msl')        
        msl = 4; 
    end
    if ~exist('msh')        
        msh = 4; 
    end
    vars.s_grid = exp(linspace(-msl*param.sigma_s+log(param.sigma_z),msh*param.sigma_s+log(param.sigma_z),param.ns)');
    if param.ns==1
        vars.s_grid=param.sigma_z;
    end
    vars.smax = max(vars.s_grid);
    vars.smin = min(vars.s_grid);
    
    
    % Create grid for z
    %
    if ~exist('mzl')        
        mzl = 3; 
    end
    if ~exist('mzh')        
        mzh = 3; 
    end
    vars.z_grid = linspace(-mzl*param.sigma_z,mzh*param.sigma_z,param.nz)';
    vars.zmax = max(vars.z_grid);
    vars.zmin = min(vars.z_grid);
    
    % Create transition matrices
    %    
    vars.pi_ys = create_ys_mat(param,vars);
    vars.pi_ysz = create_ysz_mat(param,vars);
    m_ysz=reshape(vars.pi_ysz(:),param.ny,param.ns,param.nz,param.ny,param.ns,param.nz);
    vars.pi_z=reshape(squeeze(sum(sum(m_ysz(1,:,:,:,:,:),4),5)),param.ns,param.nz,param.nz); % retrieve unconditional distribution for z1 only (given s0,z0)
    [vars.gz,~,~,~]=create_gz(param,vars);
       
    
    % Grid for n
    %
    vars.nmax = 80; %30
    vars.nmin = 0; % 0
%     vars.n_grid = linspace(vars.nmin,vars.nmax,param.nn);   
    vars.n_grid = [0,exp(linspace(log(.1),log(vars.nmax),param.nn-1))]';   
    
    
    % Some more parameters
    %
    vars.alpha_F = (F(vars.n_grid(2),param)-F(vars.n_grid(1),param))/(vars.n_grid(2)-vars.n_grid(1)); % max derivative for F
    vars.alpha_V = (1-param.beta)^(-1)*(exp(vars.ymax+vars.zmax)*vars.alpha_F); % max derivative for V
    
    
    
    
    
    
    
