% =======================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% main.m: main replication file. 
%
% Instructions: simply run this script to get all the figures from the
% paper. The results from the calibration have been saved under
% 'results/calibration.mat' to speed up replication.
% =======================================

%% ===========
%  CALIBRATION
%  ===========

% % Uncomment or load 'calibration.mat' for the direct results 
% [param,vars,res,aux,error]=calibrate(); % script that calibrates the model by fitting moments described in paper
% save results/calibration.mat

% % 'param' contains the parameters of the model
% % 'vars' contains some variables of the model (grids, bounds, transition matrices)
% % 'res' contains the solution of the model (value and policy functions)
% % 'aux' some auxiliary variables that are precalculated once and then passed on to multiple functions to save time (large index matrices, etc.)

% % instead of re-calibrating (takes a few days/weeks), load from file
% load results/calibration.mat



%% =======================================
%  SOLVING THE MODEL: BASELINE CALIBRATION
%  =======================================

% Declare and initialize parameters (from baseline calibration)

param = struct('A',1,'alpha',.85,'b', 1.402502411917219,'beta',.996,'c',1.789395096004241,'ke', 14.213281945105606,'kf',1.956256938229172,'lambda',0.366041621306308,...
        'rho_y', 0.990372641907069,'sigma_y',0.042251089970073,'rho_z',(.95)^(1/3),'sigma_z',0.532628971768815,...
        'rho_s',0.98,'sigma_s',0.131668241312217,'cor_ys',-0.4,...
        'gamma',1.59929,'m',1,...
        'ny',21,'ns',15,'nz',15,'nn',50);  
 
vars = initialize_vars(param); % required to initialize 'vars' once the parameters are known

% Compute equilibrium

[res,aux,error]=solve(param,vars);
save results/calibration;

% % instead of recomputing (15-20mn), load from file
% load results/calibration.mat

%% =======
%  TABLE 2
%  =======

% Plot the fitted moments from the calibration
compute_moments(param,vars,res);

[~,g,~,outpp]=findss_value(0,param.sigma_z,param,vars,res);
fprintf('Evaluating parameter estimates:\n');
fprintf('b / output_per_person = %2.3f\n',param.b/outpp); 

[z_id,n_id]=ndgrid(1:param.nz,1:param.nn);
output_per_firm = param.A.*exp(vars.z_grid(z_id)).*F(vars.n_grid(n_id),param);
avg_output_per_firm = sum(sum (output_per_firm.*g )) / sum(g(:));
fprintf('ke / avg_output_per_firm = %2.3f\n',param.ke/avg_output_per_firm); 
fprintf('kf / avg_output_per_firm = %2.3f\n',param.kf/avg_output_per_firm);

%% =============
%  TABLE 3 and 4
%  =============

% Display empirical business cycle statistics 
empirics();
% Display simulated business cycle statistics with both y and s shocks
stats(param,vars,res,'y+s');
% Display simulated business cycle statistics with y shocks only
stats(param,vars,res,'y only');

% To get the business cycle statistics of the standard DMP go to subfolder
% 'DMP' and run 'main()'.

%% ========
%  FIGURE 3
%  ========

firm_growth_distribution(param,vars,res);

%% ========
%  FIGURE 4
%  ========

policy(param,vars,res);

%% =================
%  FIGURE 6,7 and 10
%  =================

areagraph();

%% ===================
%  FIGURE 8, 11 and 18
%  ===================

impulse();

%% ============
%  FIGURE 9, 12
%  ============

impulse_breakdown();

%% ========================
%  FIGURE 13, 14, 15 and 17
%  ========================

counterfact(param,vars,res);


%% =================================
%  WAGE FACTS FROM ONLINE APPENDIX G
%  =================================

% Wage dispersion, cyclicality and size-wage differential (G.1 and G.2)
wage(param,vars,res);

% Beaudry and DiNardo (1991) vs Hagedorn and Manovskii (2013)
wage_extended(param,vars,res);



