% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% policy (param,vars,res): generates a graph showing the average employment
% policy of firms as a function of their growth rate.
%
% Instructions: - 'param' is a structure of parameters for the model
% - 'vars' is a structure of variables created by 'initialize_vars(param)'
% - 'res' is a structure of results for the model from 'solve(param,vars)'
%
% ========================================================================

function policy(param,vars,res)

close all;

Y0=ceil(param.ny/2);
S0=ceil(param.ns/2);
[u0,g0]=findss(Y0,S0,param,vars,res); % find ss when y is constant equal to mean
y0=vars.y_grid(Y0);
s0=vars.s_grid(S0); 

T=10; % number of quarters

% Declare all variables that we are going to keep track of
%
y=y0*ones((T+1)*3,1);
s=s0*ones((T+1)*3,1);

% draws the aggregate shocks
rng(0,'twister');
for t=2:(T+1)*3    
    [y(t),s(t)]=draw_ys_value (y(t-1),s(t-1),param,vars);     
end

% init the sample of firms
N=10000;
firm_z=zeros(N,(T+1)*3);
firm_n=zeros(N,(T+1)*3);
firm_hi=zeros(N,(T+1)*3);
firm_qu=zeros(N,(T+1)*3);
firm_lf=zeros(N,(T+1)*3);
firm_exit=zeros(N,(T+1)*3);
[firm_z0,firm_n0]=draw_firms(g0,N,param); % draw a population of N firms from the steady-state distribution
firm_z0=vars.z_grid(firm_z0);
firm_n0=vars.n_grid(firm_n0);

[Mz_id,Mn_id]=ndgrid(1:param.nz,2:param.nn);
    
fprintf('Simulating the economy [');


    for i=1:(T+1)*3
        y1=y(i);
        s1=s(i);
        
        % correcting for selection
      
        firm_z1=max(min(draw_z_value(s0,firm_z0,param),vars.zmax),vars.zmin);
        firm_n1=max(min(res.gn_spline(y1*ones(N,1),s1*ones(N,1),firm_z1,firm_n0),vars.nmax),vars.nmin);
        % hirings
        firm_hi1=res.gn_spline(y1*ones(N,1),s1*ones(N,1),firm_z1,firm_n0)-firm_n0;
        firm_hi1(firm_hi1<0)=0;
        firm_hi(:,i)=firm_hi1;
        % layoffs
        firm_lf(:,i)=res.gtau_spline(y1*ones(N,1),s1*ones(N,1),firm_z1,firm_n0).*firm_n0;
        % quits
        firm_qu1=firm_n0-res.gn_spline(y1*ones(N,1),s1*ones(N,1),firm_z1,firm_n0);
        firm_qu1=firm_qu1-firm_lf(:,i);
        firm_qu1(firm_qu1<0)=0;
        firm_qu(:,i)=firm_qu1;
        % exits
        ge = res.exit_value_spline(y1*ones(N,1),s1*ones(N,1),firm_z1,firm_n0) > res.V_spline(y1*ones(N,1),s1*ones(N,1),firm_z1,firm_n0);
        I=find(ge==true); % find the exiters
        [firm_z1_id,firm_n1_id]=draw_firms(g0,length(I),param); % replace exiters with new firms
        firm_z1(I)=vars.z_grid(firm_z1_id);
        firm_n1(I)=vars.n_grid(firm_n1_id);
        firm_z(:,i)=firm_z0;
        firm_n(:,i)=firm_n0;
        firm_exit(I,i)=1;
                
                
        [u1,g1] = distdyn_value(u0,g0,y1,s1,s0,param,vars,res);
        
        
        if(floor(10*i/((T+1)*3))~=floor(10*(i-1)/((T+1)*3))) % plot progress every 10%
            fprintf('.');
        end    
        
        if i == 3*(T+1);
            fprintf(']\n');
        end
       
        
        % Update state variables 
        g0=g1;
        u0=u1;
        s0=s1;
        firm_z0=firm_z1;
        firm_n0=firm_n1;
        
    end   
    
    
    % ------------------------------------
    % Now, take care of the simulated data
    %
    
    firm_n_q=zeros(N,T+1);
    firm_exit_q=zeros(N,T+1);
    firm_hi_q=zeros(N,T+1);    
    firm_lf_q=zeros(N,T+1);    
    firm_qu_q=zeros(N,T+1);    
    
    for i=1:(T+1)
        firm_n_q(:,i)=firm_n(:,(i-1)*3+1);
        firm_exit_q(:,i)=firm_exit(:,(i-1)*3+1)+firm_exit(:,(i-1)*3+2)+firm_exit(:,(i-1)*3+3);        
        firm_hi_q(:,i)=firm_hi(:,(i-1)*3+1)+firm_hi(:,(i-1)*3+2)+firm_hi(:,(i-1)*3+3);
        firm_lf_q(:,i)=firm_lf(:,(i-1)*3+1)+firm_lf(:,(i-1)*3+2)+firm_lf(:,(i-1)*3+3);
        firm_qu_q(:,i)=firm_qu(:,(i-1)*3+1)+firm_qu(:,(i-1)*3+2)+firm_qu(:,(i-1)*3+3);
    end
    
    % compute growth rates
    firm_growth_q=zeros(N,T);    
    firm_hi_ratio_q=zeros(N,T);    
    firm_lf_ratio_q=zeros(N,T);
    firm_qu_ratio_q=zeros(N,T);
    for i=1:T
        firm_growth_q(:,i)=200*(firm_n_q(:,i+1)-firm_n_q(:,i))./(firm_n_q(:,i+1)+firm_n_q(:,i));
        firm_hi_ratio_q(:,i)=200*firm_hi_q(:,i)./(firm_n_q(:,i+1)+firm_n_q(:,i));
        firm_lf_ratio_q(:,i)=200*firm_lf_q(:,i)./(firm_n_q(:,i+1)+firm_n_q(:,i));
        firm_qu_ratio_q(:,i)=200*firm_qu_q(:,i)./(firm_n_q(:,i+1)+firm_n_q(:,i));
    end
    firm_exit_q=firm_exit_q(:,1:T); % only exits in the 3 periods preceding observation matter (obs included) which correspond to a translation of the thing
    firm_n_q=.5*(firm_n_q(:,1:T)+firm_n_q(:,2:(T+1))); % keep average size to weight observations
    
    firm_growth_q=firm_growth_q(:);
    firm_hi_ratio_q=firm_hi_ratio_q(:);
    firm_lf_ratio_q=firm_lf_ratio_q(:);
    firm_qu_ratio_q=firm_qu_ratio_q(:);
    firm_exit_q=firm_exit_q(:);
    firm_n_q=firm_n_q(:);
    
    % remove exiting firms
    I=find(firm_exit_q==0);
    firm_exit_q=firm_exit_q(I);
    firm_growth_q=firm_growth_q(I);
    firm_hi_ratio_q=firm_hi_ratio_q(I);
    firm_lf_ratio_q=firm_lf_ratio_q(I);
    firm_qu_ratio_q=firm_qu_ratio_q(I);
    firm_n_q=firm_n_q(I);
    N=length(I);
    
    % --------------------------------------
    % Compute average policy per growth rate 
    % NB: using same bins as DFH
    
    load data/DFH_DATA;
    
    axis=100*DFH_turnover(:,1);
    nbins=length(axis);
    edges=[-inf;.5*(axis(1:nbins-1)+axis(2:nbins));inf];
    
    
    [n bin]=histc(firm_growth_q,edges);
    count=accumarray(bin,firm_n_q); %weight each growth rate by employment 
    plot_hi=accumarray(bin,firm_hi_ratio_q.*firm_n_q); %weight by employment
    plot_qu=accumarray(bin,firm_qu_ratio_q.*firm_n_q); 
    plot_lf=accumarray(bin,firm_lf_ratio_q.*firm_n_q); 
    plot_g=accumarray(bin,firm_growth_q.*firm_n_q); 
    plot_hi=plot_hi./count;
    plot_qu=plot_qu./count;
    plot_lf=plot_lf./count;
    plot_g=plot_g./count;
    
    % ----------
    % Plot graph
    
    figure('Units','inches','Position',[0,0,4,7.5]);
    set(gcf,'PaperUnits',get(gcf,'Units'),'PaperPosition',get(gcf,'Position')+[-.25 2.5 0 0]);    
    set(gcf,'DefaultAxesColorOrder',[0 0 0]);
    
    subplot(2,1,1);
    hold on
    DFH_axis=100*(DFH_turnover(:,2)-DFH_turnover(:,4)-DFH_turnover(:,5)); % I compute the average growth rate for each bin, otherwise the number don't add up to the 45 degree line
    plot(DFH_axis,100*DFH_turnover(:,2),'-','LineWidth',1,'Color',[0 0 0]);
    plot(DFH_axis,100*DFH_turnover(:,4),'-.','LineWidth',1,'Color',[0 .5 .25]);
    plot(DFH_axis,100*DFH_turnover(:,5),':','LineWidth',1,'Color',[0 .25 .75]);
%     dashline(DFH_axis,100*DFH_turnover(:,5),.5,.5,.5,.5,'LineWidth',1,'Color',[0 .25 .75]);
    plot(DFH_axis,abs(DFH_axis),'--','Color','k');
    leghdl = legend('Hirings','Quits','Layoffs','$45^\circ$');
    set(leghdl,'Color','none','Box','off','Position',[.1 .82 .4 .1],'FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
    set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
    xlim([-120 120]);
    ylim([0 150]);
    xlabel('Employment growth','FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
    ylabel('Percent of employment','FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
    title('Data','FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
    box on;
    hold off
    
    subplot(2,1,2);
    hold on
    
    plot(plot_g,plot_hi,'-','LineWidth',1,'Color',[0 0 0]);
    plot(plot_g,plot_qu,'-.','LineWidth',1,'Color',[0 .5 .25]);
    plot(plot_g,plot_lf,':','LineWidth',1,'Color',[0 .25 .75]);
%     dashline(plot_g,plot_lf,.5,.5,.5,.5,'LineWidth',1,'Color',[0 .25 .75]);
    plot(plot_g,abs(plot_g),'--','Color','k');
    xlim([-120 120]);
    ylim([0 150]);
    xlabel('Employment growth','FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
    ylabel('Percent of employment','FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
    title('Model','FontUnits','points','interpreter','latex','FontSize',9,'FontName','Times');
    set(gca,'Units','normalized','FontUnits','points','FontWeight','normal','FontSize',9,'FontName','Times');
    box on;
    hold off
    
    print('-depsc2','graphs/policy.eps');
    
