% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% simulate(y,s,param,vars,res,u0,g0,s_realized,entry_type,entry): simulates 
% the economy for a given series of aggregate shocks (y,s) and returns the
% relevant monthly time series. 
%
% Instructions: - 'y,s' two (Tx1) series of shocks to y and s (in value)
% - 'param' is a structure of parameters for the model
% - 'vars' is a structure of variables created by 'initialize_vars(param)'
% - 'res' is a structure of results for the model from 'solve(param,vars)'
% - 'u0,g0' (optional): initial unemployment rate and employment
% distribution. If not specified, start at non stochastic steady state.
% - 's_realized' (optional) series of actually realized volatility shocks.
% By default, equal to series 's'.
% - 'entry_type' (optional) is a string either equal to 'free entry', 
% 'fixed entry' or 'exogenous entry'. 'free entry' is the full model, 
% 'fixed entry' evaluates entry in the first period and then keeps it constant
% throughout (out of eqm), 'exogenous entry' takes an entry series as given
% in 'entry' and uses it to update the distribution.
%
% ========================================================================

% -------------------------------------------------------------------------
% simulate.m: simulate the economy for a given series of aggregate shocks (y,s)
%             returns the relevant monthly time series
% arguments u0 and g0 are optional and correspond to the initial
% unemployment and % distribution
%
% entry_type is either 'free entry', 'fixed entry' or 'exogenous entry'
% free entry is the full model, 'fixed entry' evaluates entry in the first
% period and then keeps it constant throughout (out of eqm), 'exogenous
% entry' takes an entry series as given in 'entry' and uses it to update
% the distribution.

function [unemp,iqr_y,std_y,outpp,v,theta,ue,eu,ee,entry,exit,hi,sp,qu,lf,output,error,avgfrmsize,prody,massfirm,cons,empl]=simulate(y,s,param,vars,res,u0,g0,s_realized,entry_type,entry)

T=length(y);

if length(s)~=T
    fprintf('%s.m: the two aggregate shock series have inconsistent sizes.\n',mfilename);
end

if ~exist('s_realized')
    s_realized=[];
end

if isempty(s_realized)
    s_realized=s;
end

if ~exist('entry_type')
    entry_type='free entry';
end

if ~strcmp(entry_type,'free entry') && ~strcmp(entry_type,'fixed entry') && ~strcmp(entry_type,'exogenous entry')
    fprintf('%s.m: unknown entry type ''%s''. Choose either ''free entry'' or ''fixed entry'' or ''exogenous entry''.\n',mfilename,entry_type);
    return;
end

if ~exist('entry')
    entry=[];
end


% Variable declaration
unemp=zeros(T,1);
output=zeros(T,1);
outpp=zeros(T,1);
theta=zeros(T,1);
v=zeros(T,1);
if isempty(entry)
    entry=zeros(T,1);
end
exit=zeros(T,1);
hi=zeros(T,1);
sp=zeros(T,1);
qu=zeros(T,1);
lf=zeros(T,1);
ue=zeros(T,1);
eu=zeros(T,1);
ee=zeros(T,1);
avgfrmsize=zeros(T,1);
prody=zeros(T,1);
massfirm=zeros(T,1);
cons=zeros(T,1);
empl=zeros(T,1);

% init
if nargin<6 % u0 and g0  not specified
    [u0,g0]=findss_value(y(1),s(1),param,vars,res);
end

s0=s(1);
% s0_id=s_id(1);
s0_realized=s0;

% Draw a population of firms to compute IQR
rng(0,'twister'); % reset the random number generator seed, so that the function
% produces consistent results
N=10000;
firm_z=zeros(N,T);
[firm_z0,~]=draw_firms(g0,N,param); % draw a population of N firms from the steady-state distribution
firm_z0=vars.z_grid(firm_z0);

[Mz_id,Mn_id]=ndgrid(1:param.nz,2:param.nn);

fprintf('[');


for i=1:T
    y1=y(i);
    s1=s(i);
    
    firm_z1=draw_z_value(s0,firm_z0,param);
    firm_z(:,i)=firm_z1;
    
    switch entry_type
        case 'free entry'
            [u1,g1,outpp(i),v(i),theta(i),ue(i),eu(i),ee(i),entry(i),exit(i),hi(i),sp(i),qu(i),lf(i),output(i),error,cons(i)] = distdyn_value(u0,g0,y1,s1,s0_realized,param,vars,res);
        case 'fixed entry'
            if i==1
                [u1,g1,outpp(i),v(i),theta(i),ue(i),eu(i),ee(i),entry(i),exit(i),hi(i),sp(i),qu(i),lf(i),output(i),error,cons(i),entry_pattern] = distdyn_value(u0,g0,y1,s1,s0_realized,param,vars,res);
                fixed_entry=entry(1); % for fixed entry, initialize the fixed entry to first iteration
            else
                [u1,g1,outpp(i),v(i),theta(i),ue(i),eu(i),ee(i),entry(i),exit(i),hi(i),sp(i),qu(i),lf(i),output(i),error,cons(i)] = distdyn_value_fixed_entry_pattern(u0,g0,y1,s1,s0_realized,param,vars,res,fixed_entry,entry_pattern);
            end
%         case 'exogenous entry'            
%             [u1,g1,outpp(i),v(i),theta(i),ue(i),eu(i),ee(i),~,exit(i),hi(i),sp(i),qu(i),lf(i),output(i),error,cons(i)] = distdyn_value_fixed_entry(u0,g0,y1,s1,s0_realized,param,vars,res,entry(i));
    end

    unemp(i)=u1;
    
    avgfrmsize(i)=sum(g1)*vars.n_grid/sum(sum(g1));
    prody(i)=sum(sum((exp(y1+vars.z_grid(Mz_id)).*param.A.*param.alpha.*vars.n_grid(Mn_id).^(param.alpha-1)).*g1(Mz_id+(Mn_id-1)*param.nz)))/sum(g1(:));
    massfirm(i)=sum(g1(:));
    empl(i)=sumpop(g1,param,vars);    
    
    
    if(floor(10*i/T)~=floor(10*(i-1)/T)) % plot progress every 10%
        fprintf('.');
    end
    
    if i == T
        fprintf('] ');
    end
    
    
    % Update state variables
    g0=g1;
    u0=u1;
    s0=s1;
    s0_realized=s_realized(i);
    firm_z0=firm_z1;
    
end


% ---------------------------------------
% Compute the IQR and STD measures of TFP
%

% definitions
T_y=floor(T/12);
iqr_y=zeros(T_y-1,1);
std_y=zeros(T_y-1,1);

% retrieve yearly data
z_y=firm_z(:,(1:T_y)*12); % take observations in the month of december every year


% regress z_t on z_{t-1}, year fixed effect and firm fixed effect

A=z_y(1:N*(T_y-1))'; % dependent variable
B=z_y(N+1:N*T_y)'; % first regressor: z_{t-1}
year = reshape(repmat(reshape(speye(T_y-1),1,(T_y-1)^2),[N 1]),N*(T_y-1),T_y-1); % year fixed effect
firm = repmat(speye(N),[T_y-1 1]); % firm fixed effect

C=[B year];
beta=C\A; % regress A on C
rho=beta';




% now, recover a measure of the shock e_hat to compute IQR/STD
firm = speye(N);
for i=2:T_y
    year = sparse(1:N,(i-1)*ones(N,1),ones(N,1),N,T_y-1);
%     e_hat=firm_z(:,i*12)-[firm_z(:,i*12-12) year firm]*rho';
    e_hat=firm_z(:,i*12)-[firm_z(:,i*12-12) year]*rho';
    iqr_y(i-1)=iqr(e_hat);
    std_y(i-1)=std(e_hat);
end

end

