% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% solve(param,vars,res,aux,eqm_type): solves the model. 
%
% Instructions: - 'param' is a structure of parameters for the model
% - 'vars' is a structure of variables created by 'initialize_vars(param)'
% - 'res' (optional) is a previous 'result' structure to be used as initial seed
% - 'aux' (optional) is a previous 'aux' structure to be used to avoid recomputing stuff
% - 'eqm_type' string either equal to 'GE' (default) or 'PE' for general eqm vs partial eqm
%
% ========================================================================

function [res,aux,error]=solve(param,vars,res,aux,eqm_type)
% Recover optional arguments
if nargin<3
    res=[];
end

if nargin<4
    aux=[];
end

if nargin<5
    eqm_type='GE'; % choose 'GE' or 'PE' for general or partial equilibrium
end

% ---------------------------------
% Declare and initialize parameters
%


if ~strcmp(eqm_type,'GE') && ~strcmp(eqm_type,'PE')
    fprintf('%s.m: unknown equilibrium type ''%s''. Choose either ''GE'' or ''PE''.\n',mfilename,eqm_type);
    return;
end

CV_tol=1e-5;

if isempty(res)
    res = struct('V',[],'U',[],'ge',[],'gn',[],'gne',[],'gtau',[],...
        'gtheta_x',[],'gtheta_xu',[],'gx',[],'gxu',[],'gn_id',[],...
        'gn_spline',[],'V_spline',[],'exit_value_spline',[],...
        'gtheta_x_spline',[],'gtau_spline',[],'gtheta_xu_spline',[],...
        'cost',[],'entering_firmsize_id',[],'exit_value',[],'M',[]);
end


% Define some useful index matrices that we can keep over iterations
%
if isempty(aux) || ~isequal(size(aux.y_id),[param.ny*param.ns*param.nz*param.nn param.nn])
    aux = struct('y_id',[],'s_id',[],'z_id',[],'n_id',[],'nhat_id',[],...
        'Mn',[],'Mnhat',[],'Mne',[],'ys_id',[],...
        'mn',[],'mnhat',[],'r',[],'ratio',[],'ratio_id',[],'nb_ratio',[],...
        'ry_id',[],'rs_id',[],'rr_id',[],'mr',[],'Mr_id',[],...
        'net_production',[],'theta_x_r',[],'ysr_id',[],'H_prime_inv',[]);
    
    [aux.y_id,aux.s_id,aux.z_id,aux.n_id,aux.nhat_id]=ndgrid(1:param.ny,1:param.ns,1:param.nz,1:param.nn,1:param.nn);
    aux.y_id=reshape(aux.y_id,param.ny*param.ns*param.nz*param.nn,param.nn);
    aux.s_id=reshape(aux.s_id,param.ny*param.ns*param.nz*param.nn,param.nn);
    aux.z_id=reshape(aux.z_id,param.ny*param.ns*param.nz*param.nn,param.nn);
    aux.n_id=reshape(aux.n_id,param.ny*param.ns*param.nz*param.nn,param.nn);
    aux.nhat_id=reshape(aux.nhat_id,param.ny*param.ns*param.nz*param.nn,param.nn);
    
    aux.Mn=vars.n_grid(aux.n_id);
    aux.Mnhat=vars.n_grid(aux.nhat_id);
    aux.Mne=aux.Mnhat-aux.Mn; % compute corresponding job creation
    aux.Mne(aux.Mne<0)=0; % for n_hat<=n n_e = 0
    
    aux.ys_id=aux.y_id + (aux.s_id-1)*param.ny;
    
    
    % Do some precalculation for the trade-off between tau and x
    % Solve: max tau*U+(1-tau)*p(x)x s.t (1-tau)(1-lambda*p(x))=r=nhat=n
    %
    
    [aux.mn,aux.mnhat]=ndgrid(vars.n_grid,vars.n_grid);
    aux.r = aux.mnhat./aux.mn;
    aux.r(aux.r>1 | isnan(aux.r))=1; % for all r>1, ne>0 so tau=0 p(x)=0 so set r=1
    [aux.ratio,~,aux.ratio_id]=unique(aux.r(:));
    aux.nb_ratio=length(aux.ratio);
    aux.ratio_id=reshape(aux.ratio_id,param.nn,param.nn); % r=ratio(ratio_id)
    
    [aux.ry_id,aux.rs_id,aux.rr_id]=ndgrid(1:param.ny,1:param.ns,1:aux.nb_ratio);
    aux.ry_id=reshape(aux.ry_id,param.ny*param.ns,aux.nb_ratio);
    aux.rs_id=reshape(aux.rs_id,param.ny*param.ns,aux.nb_ratio);
    aux.rr_id=reshape(aux.rr_id,param.ny*param.ns,aux.nb_ratio);
    
    
    aux.mr=aux.ratio(aux.rr_id);
    aux.mr=reshape(aux.mr,[param.ny*param.ns aux.nb_ratio]); % needed for compatibility with ny*ns=1
    aux.Mr_id=aux.ratio_id(aux.n_id+(aux.nhat_id-1)*param.nn);
    aux.ysr_id=aux.y_id + (aux.s_id-1)*param.ny+(aux.Mr_id-1)*param.ny*param.ns;
    
    % Precalculations for splines
    aux.H_prime_inv = spline_precalc(vars.n_grid);
    
end

% Some more precalculations that must be done everytime
aux.net_production = exp(vars.y_grid(aux.y_id(:,1))+vars.z_grid(aux.z_id(:,1))).*F(vars.n_grid(aux.n_id(:,1)),param)-param.kf;
aux.theta_x_r=pinv((1-aux.ratio(aux.rr_id))./param.lambda);% put p^-1((1-r)/lambda) in this matrix
aux.theta_x_r=reshape(aux.theta_x_r,[param.ny*param.ns aux.nb_ratio]); % needed for compatibility with ny*ns=1

% Initial conditions
%
if isempty('res.V') || ~isequal(size(res.V),[param.ny*param.ns*param.nz*param.nn 1])
    % since U < cost in equilibrium, start with a cost just above
    % b/(1-beta) which is the lower bound of U
    cost = 1.1*param.b/(1-param.beta)*ones(param.ny*param.ns,1);
%     res.V = zeros(param.ny,param.ns,param.nz,param.nn);
    res.V = aux.net_production+param.beta/(1-param.beta)*(exp(vars.y_grid(ceil(param.ny/2))+vars.z_grid(ceil(param.nz/2)))*F(vars.n_grid(aux.n_id(:,1)),param)-param.kf); % initializing away from zero helps
else
    cost = res.cost;
    entering_firmsize_id=res.entering_firmsize_id; % prevents a bug when doing PE
end



% Define our main value functions
V1=res.V(:);

% --------------
% Main loop on V
%

counter=0;
tic;
nb_successes=0; % count number of iterations below convergence treshold CV_tol
date_string = datestr(now,'mm-dd.HHMM'); % retrieve date to label and save results
convergence = inf; % measure of convergence
time_0=clock();
error=0;

% Nb: prevent algorithm to stop at the beginning with less than 200
% iterations, must be below convergence threshold for a number of 25
% iterations to prevent from stopping too early
%     while((convergence>CV_tol || counter<200 || nb_successes <5) && counter<1500)
while((convergence>CV_tol || nb_successes <5) && counter<1500)
    
    if (convergence>CV_tol)
        nb_successes=0;
    else
        nb_successes=nb_successes+1;
    end
    
    if(mod(counter,100)==0) % every 100 iteration
        time_iter=clock; % keep time in memory to compute speed
    end
    
    V0=V1;
    
    % First, solve for the value of unemployment
    %
    if strcmp(eqm_type,'GE') || counter==0 % compute the value of unemployment if GE or first iteration of PE
        [U,gxu,gtheta_xu,errorU] = solveU(param,vars,cost);
        error=error+8*errorU;
    end
    
    
    % Now solve for optimal firm policy given aggregate state variable
    % \hat{y},\hat{\sigma}
    %
    
    % Substitute the precalculated results from the problem:
    % max_{x,tau} tau*U+(1-tau)*p(x)x s.t (1-tau)(1-lambda*p(x))=r=nhat/n
    % in the grid (n,nhat) grid to avoid solving the same problem over and over.
    
    theta_x_opt = solvethetax(param,vars,cost,U);
    
    mtheta_x_opt=theta_x_opt;
    mtheta_x_opt=min(cat(3,mtheta_x_opt(:,ones(aux.nb_ratio,1)),aux.theta_x_r),[],3);
    mp=p(mtheta_x_opt,param);
    mx=cost(:,ones(aux.nb_ratio,1))-param.c./q(mtheta_x_opt,param);
    
    
    mtau=1-aux.mr./(1-param.lambda*mp); % for each (y,s,r), compute corresponding tau given constraint
    MUx=mtau.*U(:,ones(aux.nb_ratio,1))+(1-mtau).*param.lambda.*mp.*mx;
    MUx(mtau<0)=-inf; % exclude non-admissible points: no solution for r>1-lambda_e*p
    
    
    % We can now solve the firm's maximization problem for each (y,s,z,n)
    % over nhat, x and tau.
    %
    % On a grid (y*s*z*n,nhat):
    MUx=MUx(aux.ysr_id);
    Vex=repmat(reshape(V0,param.ny*param.ns*param.nz,param.nn),[param.nn 1]); % optimized version of Vex=V0(y_id + (s_id-1)*ny+(z_id-1)*ny*ns+(nhat_id-1)*ny*ns*nz)
    M=aux.Mn.*MUx-cost(aux.ys_id).*aux.Mne + Vex; % objective function
    pp=spline(vars.n_grid,M,aux.H_prime_inv);
    [Vex,In]=max(M,[] ,2); %max over \hat{n}
    
    % Spline interpolation on two adjacent segments:
    % use the spline to compute the max in the two intervals
    % surrounding the optimum
    subint=max(1,In-1);
    superint=min(param.nn-1,In);
    root_int=[subint subint superint superint];
    
    % extract the two relevant polynomials
    vec=(1:param.ny*param.ns*param.nz*param.nn)'+(3*(subint-1)+1-1)*param.ny*param.ns*param.nz*param.nn;
    ppsub=pp(horzcat(vec,vec+param.ny*param.ns*param.nz*param.nn,vec+2*param.ny*param.ns*param.nz*param.nn));
    vec=(1:param.ny*param.ns*param.nz*param.nn)'+(3*(superint-1)+1-1)*param.ny*param.ns*param.nz*param.nn;
    ppsup=pp(horzcat(vec,vec+param.ny*param.ns*param.nz*param.nn,vec+2*param.ny*param.ns*param.nz*param.nn));
    
    nsub=vars.n_grid(subint);
    nsup=vars.n_grid(superint);
    
    Msub=M((1:param.ny*param.ns*param.nz*param.nn)'+(subint-1)*param.ny*param.ns*param.nz*param.nn);
    Msup=M((1:param.ny*param.ns*param.nz*param.nn)'+(superint-1)*param.ny*param.ns*param.nz*param.nn);
    
    % define the coefficients of the polynomials' derivative
    Asub=3*ppsub(:,3);
    Bsub=2*ppsub(:,2)-6*ppsub(:,3).*nsub;
    Csub=ppsub(:,1)+nsub.*(-2*ppsub(:,2)+3*ppsub(:,3).*nsub);
    
    Asup=3*ppsup(:,3);
    Bsup=2*ppsup(:,2)-6*ppsup(:,3).*nsup;
    Csup=ppsup(:,1)+nsup.*(-2*ppsup(:,2)+3*ppsup(:,3).*nsup);
    
    % solve the quadratic equation
    Deltasub=Bsub.^2-4*Asub.*Csub;
    Deltasup=Bsup.^2-4*Asup.*Csup;
    
    Inegsub=Deltasub<0;
    Deltasub(Inegsub)=0;
    Inegsup=Deltasup<0;
    Deltasup(Inegsup)=0;
    
    % Compute the possible extrema in each interval: the two roots of
    % the derivative and the corners
    n_roots = horzcat(-(Bsub-sqrt(Deltasub))./(2*Asub),-(Bsub+sqrt(Deltasub))./(2*Asub),...
        -(Bsup-sqrt(Deltasup))./(2*Asup),-(Bsup+sqrt(Deltasup))./(2*Asup));
    
    
    % Evaluate the spline at each possible extremum
    
    V_roots=horzcat(Msub+(n_roots(:,1)-nsub).*(ppsub(:,1)+(n_roots(:,1)-nsub).*(ppsub(:,2)+(n_roots(:,1)-nsub).*ppsub(:,3))),...
        Msub+(n_roots(:,2)-nsub).*(ppsub(:,1)+(n_roots(:,2)-nsub).*(ppsub(:,2)+(n_roots(:,2)-nsub).*ppsub(:,3))),...
        Msup+(n_roots(:,3)-nsup).*(ppsup(:,1)+(n_roots(:,3)-nsup).*(ppsup(:,2)+(n_roots(:,3)-nsup).*ppsup(:,3))),...
        Msup+(n_roots(:,4)-nsup).*(ppsup(:,1)+(n_roots(:,4)-nsup).*(ppsup(:,2)+(n_roots(:,4)-nsup).*ppsup(:,3))));
    
    
    % Reject inactive guys
    % Notice that I reject inactive guys because there is a kink in the
    % value function. Therefore, whenever a firm prefers to stay where it
    % is, I don't use a spline and assume that the optimum is reached in
    % inaction. This is equivalent to doing a spline to the left and to the
    % right of n, assuming that there is a break at n and imposing the
    % typical boundary conditions for the spline:
    % P'_left=(M(n)-M(n-1))/(n_grid(n)-n_grid(n-1)) and idem to the right
    % (in that case, there is no way a maximum could be reached in a nearby
    % interval because of the general concavity of M).
    inactive=In==aux.n_id(:,1);
    V_roots(inactive(:,ones(4,1)))=-inf;   %optimized verison of V_roots(Inegsup,1:4)=-inf (faster than repmat,etc)
    
    % Reject imaginary solutions
    V_roots(Inegsup(:,ones(4,1)))=-inf;    %optimized verison of V_roots(Inegsup,1:4)=-inf
    
    
    % Check that the roots are with the bounds of the intervals,
    % otherwise ignore
    V_roots(n_roots < vars.n_grid(root_int) | n_roots > vars.n_grid(root_int+1))=-inf;
    
    % Maximize over the roots
    V_roots=horzcat(V_roots,Vex);
    [V,groots]=max(V_roots,[],2);
    
    % Check exits
    %
    exit_value=aux.Mn(:,1).*U(aux.y_id(:,1) + (aux.s_id(:,1)-1)*param.ny);
    cont_value=V;
    Vex=max(cont_value,exit_value);
    
    
    % Compute expectation and update value function
    %
    
    V_reshaped=reshape(Vex,param.ny*param.ns*param.nz,param.nn);
    expectation=reshape(vars.pi_ysz*V_reshaped,param.ny*param.ns*param.nz*param.nn,1);
    V1=aux.net_production+param.beta*expectation;
    
    % Now, solve the free entry condition
    if strcmp(eqm_type,'GE')
        [cost,entering_firmsize_id,error_fe] = solvetheta(param,vars,V1);
        error=error+error_fe;
        if error
            return;
        end
    end
    
    
    
    % Update measure of convergence
    convergence = mean(abs((V1(:)-V0(:))./V0(:)));
    
    
    counter=counter+1;
    if(mod(counter,100)==0) % every 10 iterations, display convergence
        fprintf('Iteration No.%d - CV=%d\n',counter,convergence);
    end
    
    if mod(counter,500)==10 % show time needed for 10 iterations every 100 iteration
        fprintf('Time needed for 1 iterations: %f seconds\n',etime(clock,time_iter)/10);
    end
    
end

if convergence<CV_tol
    fprintf('Convergence reached in %5.1f seconds and %d iterations.\n',etime(clock,time_0),counter);
else
    fprintf('Convergence failed after %5.1f seconds and %d iterations.\n',etime(clock,time_0),counter);
    %error=error+16;
end

% -------------------------
% Retrieve optimal policies
%
res.V = V1;
res.cost = cost;
res.entering_firmsize_id = entering_firmsize_id;
res.U = U;
res.gxu = gxu;
res.gtheta_xu = gtheta_xu;
res.exit_value = exit_value;
res.cont_value = cont_value;
res.M = M;

n_roots=horzcat(n_roots,vars.n_grid(In));
res.gn=gather(n_roots((1:param.ny*param.ns*param.nz*param.nn)'+(groots-1)*param.ny*param.ns*param.nz*param.nn));
res.gn_id=gather(In);
res.gne=(res.gn-vars.n_grid(aux.n_id(:,1))).*(res.gn-vars.n_grid(aux.n_id(:,1))>0);
gr=gather(res.gn)./vars.n_grid(aux.n_id(:,1));
res.gtheta_x=zeros(param.ny*param.ns*param.nz*param.nn,1);
res.gtau=zeros(param.ny*param.ns*param.nz*param.nn,1);
Ibelow=gr<1;
mtheta_x_opt=repmat(theta_x_opt,[param.nz*param.nn  1]);
res.gtheta_x(Ibelow)=min(horzcat(mtheta_x_opt(Ibelow),pinv((1-gr(Ibelow))./param.lambda)),[],2);
res.gx=repmat(gather(cost),[param.nz*param.nn 1])-param.c./q(res.gtheta_x,param);
res.gtau(Ibelow)=1-gr(Ibelow)./(1-param.lambda*p(res.gtheta_x(Ibelow),param));
res.ge=V<exit_value;


% Format policies
res.gn=reshape(res.gn,param.ny,param.ns,param.nz,param.nn);
res.gn_id=reshape(res.gn_id,param.ny,param.ns,param.nz,param.nn);
res.gtheta_x=reshape(res.gtheta_x,param.ny,param.ns,param.nz,param.nn);
res.gtheta_xu=reshape(res.gtheta_xu,param.ny,param.ns);
res.gx=reshape(res.gx,param.ny,param.ns,param.nz,param.nn);
res.gtau=reshape(res.gtau,param.ny,param.ns,param.nz,param.nn);
res.ge=reshape(res.ge,param.ny,param.ns,param.nz,param.nn);
res.gne=reshape(res.gne,param.ny,param.ns,param.nz,param.nn);
res.gxu=reshape(res.gxu,param.ny,param.ns);
res.entering_firmsize_id=reshape(res.entering_firmsize_id,param.ny,param.ns,param.nz);

% Precalculate interpolating splines
[y_id,s_id,z_id,n_id]=ndgrid(1:param.ny,1:param.ns,1:param.nz,1:param.nn);
res.gn_spline=griddedInterpolant(vars.y_grid(y_id),vars.s_grid(s_id),vars.z_grid(z_id),vars.n_grid(n_id),res.gn,'linear','none');
res.V_spline=griddedInterpolant(vars.y_grid(y_id),vars.s_grid(s_id),vars.z_grid(z_id),vars.n_grid(n_id),reshape(res.V,param.ny,param.ns,param.nz,param.nn),'linear','none');
res.exit_value_spline=griddedInterpolant(vars.y_grid(y_id),vars.s_grid(s_id),vars.z_grid(z_id),vars.n_grid(n_id),reshape(res.exit_value,param.ny,param.ns,param.nz,param.nn),'linear','none');
res.cont_value_spline=griddedInterpolant(vars.y_grid(y_id),vars.s_grid(s_id),vars.z_grid(z_id),vars.n_grid(n_id),reshape(res.cont_value,param.ny,param.ns,param.nz,param.nn),'linear','none');
res.gtheta_x_spline=griddedInterpolant(vars.y_grid(y_id),vars.s_grid(s_id),vars.z_grid(z_id),vars.n_grid(n_id),reshape(res.gtheta_x,param.ny,param.ns,param.nz,param.nn),'linear','none');
res.gtau_spline=griddedInterpolant(vars.y_grid(y_id),vars.s_grid(s_id),vars.z_grid(z_id),vars.n_grid(n_id),reshape(res.gtau,param.ny,param.ns,param.nz,param.nn),'linear','none');
res.gtheta_xu_spline=griddedInterpolant(vars.y_grid(y_id(:,:,1,1)),vars.s_grid(s_id(:,:,1,1)),reshape(res.gtheta_xu,param.ny,param.ns),'linear','none');

%save(sprintf('solve-%s.mat',date_string));


