% -----------------------------------------------------------------
% solvetheta.m: solves for the equilibrium market tightness on each
% submarket (y,s,x) given V and returns the corresponding matrix
% uses a tiny search algorithm that converges in log2(n)
%
function [cost,entering_firmsize_id,error] = solvetheta(param,vars,V)

error=false; % error=true if no solution

cost = zeros(param.ny*param.ns,1);
entering_firmsize_id = zeros(param.ny*param.ns,param.nz);

V=gather(V);
V=reshape(V,param.ny*param.ns,param.nz,param.nn);


% ------------------------------------------------------------
% Look for the unique cost that yield max expected profits = k
%

costmin=param.c; % lowest possible cost is c for 1 vacancy with proba 1, lowest wage
costmax=vars.alpha_V;

kl=costmin*ones(param.ny*param.ns,param.nz,param.nn);
ku=costmax*ones(param.ny*param.ns,param.nz,param.nn);


% First, test if max(V(n)-costmin*n)<ke, in which case there is no solution
%
[Mys,Mz,Mn]=ndgrid(1:param.ny*param.ns,1:param.nz,vars.n_grid);
weight=vars.gz(Mz(:,:,1));
weight=reshape(weight,[param.ny*param.ns param.nz]); % needed for compatibility with ny*ns=1
[maxval,I] = max(V - kl.*Mn,[],3);
Ineg = find(maxval < 0);
maxval(Ineg)=0;
I(Ineg)=-1; %set entering_firmsize_id=-1 if firm exits immediately
maxval=sum(maxval.*weight,2);
J=find(maxval<param.ke);

mask = ones(param.ny*param.ns,1); % mask to ignore the (y,s)'s that have no solution
if ~isempty(J)
    for i=1:length(J) % J is a vector even if maxval is 2-dimensional
        fprintf(1,'\n%s.m: y,s=%d firm entry [failed] (V<ke) - min=%d, max=%d\n',mfilename,J(i),V(J(i)),maxval(J(i)));
        cost(J(i)) = costmin; %choose the lowest cost
        mask(J(i)) = 0;
        entering_firmsize_id(J(i),:) = I(J(i),:); % Nb:entering_firmsize_id contains the index of size, actual size is n_grid(entering_firmsize_id)
        error=1;
    end
end

% Second, test if EV(0)^+>ke, in which case there is no solution either
%
mat=V(:,:,1);
mat(mat<0)=0;
mat=sum(mat.*weight,2);
J=find(mat>param.ke);

if ~isempty(J)
    for i=1:length(J)
        fprintf(1,'\n%s.m: y,s=%d firm entry [failed] (V(0)>ke) - min=%d, max=%d\n',mfilename,J(i),mat(J(i)),V(J(i),param.nz,param.nn)-vars.n_grid(param.nn)*costmin);
        mask(J(i)) = 0;
        cost(J(i)) = (costmin+costmax)/2; % cost in terms of cost (not index)
        entering_firmsize_id(J(i),:) = 1; % firm size in terms of index
        % Nb: set entering_firmsize_id to 0 in order to stabilize the algorithm (error
        % does not explode
        error=error+2;
    end
end


% -----------------------------------------------------------------
% Main loop: find the solution to E[max V-n*]^+ = ke when it exists
%

val=inf*ones(param.ny*param.ns,1);
counter=0;
current_cost=(ku+kl)/2;

while(sum(mask.*(val-param.ke).^2)/(param.ny*param.ns)>1e-9 && counter <100)
    
    current_cost=(ku+kl)/2;
    [val,I] = max(V - current_cost.*Mn,[],3); % max over n    
    Ineg = find(val < 0); % find the exiters
    val(Ineg) = 0;
    I(Ineg) = -1; % set -1 if firm exits
    val=sum(val.*weight,2); % sum over z

    for i=1:param.ny*param.ns        
            if val(i)>=param.ke
                kl(i,:,:)=(ku(i,1,1)+kl(i,1,1))/2;
            else
                ku(i,:,:)=(ku(i,1,1)+kl(i,1,1))/2;
            end        
    end

    counter=counter+1;
end
% counter

J=find(mask==1); % find the (y,s)'s that admit a solution
cost(J)=current_cost(J);
for i=1:length(J)
    entering_firmsize_id(J(i),:)=I(J(i),:); % retrieve optimal firm size of entrants
end


if counter==50
    fprintf('\n%s.m: max number of iterations reached.\n',mfilename);
    error=error+4;
end

