% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% stats (param,vars,res): 
% computes and displays standard business cycle statistics for the model.
%
% Instructions: - 'param' is a structure of parameters for the model
% - 'vars' is a structure of variables created by 'initialize_vars(param)'
% - 'res' is a structure of results for the model from 'solve(param,vars)'
% - 'opt' is a string either equal to 'y+s' (default) or 'y only' 
%
% ========================================================================

function [moments,err]=stats(param,vars,res,opt)
% Recover option
if nargin<4
    opt='y+s';
end

if ~(strcmp(opt,'y+s')|strcmp(opt,'y only'))
    error('%s.m: option ''%s'' unrecognized. Must be ''y+s'' or ''y only''.\n',mfilename,opt);
end    

% Declare stuff
Y0=ceil(param.ny/2);
S0=ceil(param.ns/2);
[u0,g0]=findss(Y0,S0,param,vars,res); % find ss when y is constant equal to mean
y0=vars.y_grid(Y0);
s0=vars.s_grid(S0); 


T=1000; % number of years
lag=1; % max number of lags in years

% Declare all variables that we are going to keep track of
%
y=y0*ones((T+lag)*12,1);
s=s0*ones((T+lag)*12,1);

% draws the aggregate shocks
rng(0,'twister');
for t=2:(T+lag)*12
    if strcmp(opt,'y only')
        [y(t),~]=draw_ys_value (y(t-1),s(t-1),param,vars); % use for the RBC stats (y only)
    else
        [y(t),s(t)]=draw_ys_value (y(t-1),s(t-1),param,vars); % use for the RBC stats (y+s)
    end    
end

% Start simulation

fprintf('\nComputing statistics ');

[unemp,iqr_y,std_y,outpp,v,theta,ue,eu,ee,entry,exit,hi,sp,qu,lf,output,err,avgfrmsize,prody] = simulate(y,s,param,vars,res,u0,g0);
    

% ---------------------
% Format simulated data

% Get rid of the first 12 months with incomplete history
y=y(lag*12+1:(lag+T)*12);
s=s(lag*12+1:(lag+T)*12);
unemp=unemp(lag*12+1:(lag+T)*12);
outpp=outpp(lag*12+1:(lag+T)*12);
output=output(lag*12+1:(lag+T)*12);
v=v(lag*12+1:(lag+T)*12);
theta=theta(lag*12+1:(lag+T)*12);
ue=ue(lag*12+1:(lag+T)*12);
eu=eu(lag*12+1:(lag+T)*12);
ee=ee(lag*12+1:(lag+T)*12);
entry=max(entry(lag*12+1:(lag+T)*12),0);
exit=exit(lag*12+1:(lag+T)*12);
hi=hi(lag*12+1:(lag+T)*12);
sp=sp(lag*12+1:(lag+T)*12);
qu=qu(lag*12+1:(lag+T)*12);
lf=lf(lag*12+1:(lag+T)*12);


% Compute quarterly averages
y_q=averageM2Q(y);
s_q=averageM2Q(s);
unemp_q=averageM2Q(unemp);
outpp_q=averageM2Q(outpp);
output_q=averageM2Q(output);
theta_q=averageM2Q(theta);
v_q=averageM2Q(v);
ue_q=averageM2Q(ue);
eu_q=averageM2Q(eu);
ee_q=averageM2Q(ee);
entry_q=averageM2Q(entry);
exit_q=averageM2Q(exit);
hi_q=averageM2Q(hi);
sp_q=averageM2Q(sp);
qu_q=averageM2Q(qu);
lf_q=averageM2Q(lf);



% Compute log deviations from trend
%


logoutpp_dev = logdev(outpp_q,'Q'); 
logoutput_dev = logdev(output_q,'Q'); 
logiqr_dev = logdev(iqr_y,'Y'); 
logstd_dev = logdev(std_y,'Y'); 
logunemp_dev = logdev(unemp_q,'Q'); 
logv_dev = logdev(v_q,'Q'); 
logtheta_dev = logdev(theta_q,'Q'); 
logue_dev = logdev(ue_q,'Q');
logeu_dev = logdev(eu_q,'Q'); 
logee_dev = logdev(ee_q,'Q'); 
logentry_dev = logdev(entry_q,'Q'); 
logexit_dev = logdev(exit_q,'Q'); 
loghi_dev = logdev(hi_q,'Q'); 
logsp_dev = logdev(sp_q,'Q'); 
logqu_dev = logdev(qu_q,'Q'); 
loglf_dev = logdev(lf_q,'Q'); 


% Compute empirical correlation
%

cov=0;
for i=1:T*4    
    x = [logoutput_dev(i);logoutpp_dev(i);logunemp_dev(i);logv_dev(i);loghi_dev(i);logqu_dev(i);loglf_dev(i)];
    cov = cov + x*x'/(T*4-1);    
end

N=length(x);
stdev=zeros(1,N);
cor=zeros(N,N);
for i=1:N;
    stdev(i)=sqrt(cov(i,i));
    for j=1:N
        cor(i,j)=cov(i,j)/sqrt(cov(i,i)*cov(j,j));
    end
end


% Display statistics
%

format short
fprintf('\n----------------------------------------------------------------------\n');
fprintf('       MODEL: Summary of the statistics (stdev(log),cor(log))\n');
fprintf('----------------------------------------------------------------------\n');
fprintf('       (1)       (2)       (3)       (4)       (5)       (6)       (7)\n');
fprintf('    output     outpp         u         v        hi        qu        lf\n');
disp(stdev);
fprintf('----------------------------------------------------------------------\n');
disp(cor);
fprintf('----------------------------------------------------------------------\n');
format long


