% =======================================================================
% UNCERTAINTY AND UNEMPLOYMENT
% by Edouard Schaal
% -----------------
% Replication files
%
% wage_extended(param,vars,res).m: based on wage.m, simulates a population of workers and 
% compute wage, then check Beaudry and DiNardo (1991) facts
%
% Instructions: - 'param' is a structure of parameters for the model
% - 'vars' is a structure of variables created by 'initialize_vars(param)'
% - 'res' is a structure of results for the model from 'solve(param,vars)'
%
% ========================================================================


function wage_extended(param,vars,res)
close all
Y0=ceil(param.ny/2);
S0=ceil(param.ns/2);
[U0,G0]=findss(Y0,S0,param,vars,res);
u0=U0;
g0=G0;


% ---------------------
% some precalculations
% 
% fprintf('Precalculations:\n');
% fprintf('Computing promised_utility [');
% 
% promised_utility = zeros(param.ny,param.ns,param.nz,param.nn);
% for y0=1:param.ny
%     for s0=1:param.ns
%         for z0=1:param.nz
%             for n0=1:param.nn
%                 promised_utility(y0,s0,z0,n0) = findW(y0,s0,z0,n0,param,vars,res); % note that promised_utility is the beta*W' in the paper
%             end
%         end        
%     end
%     fprintf('.');
% end
% 
% fprintf(']\n');
% fprintf('Computing expected utility [');
% 
% [My_id,Ms_id,Mz_id] = ndgrid(1:param.ny,1:param.ns,1:param.nz);
% worker_expected_utility = zeros(param.ny,param.ns,param.nz,param.nn); %compute worker's expected utility excluding current wage
% 
% for n=1:param.nn
%     M = (res.ge(:,:,:,n) + (1-res.ge(:,:,:,n)).*res.gtau(:,:,:,n)).*res.U(My_id + (Ms_id-1)*param.ny) + (1-res.ge(:,:,:,n)).*(1-res.gtau(:,:,:,n)).*( promised_utility(:,:,:,n) + param.lambda*p(res.gtheta_x(:,:,:,n),param).*(res.gx(:,:,:,n)-promised_utility(:,:,:,n)));
%     expectations = param.beta*vars.pi_ysz*M(:);
%     worker_expected_utility(:,:,:,n) = reshape(expectations,param.ny,param.ns,param.nz);
%     fprintf('.');
% end
% 
% fprintf(']\n');
% 
% save('results/wage.mat');

load('results/wage.mat');

% ----------------------
% Initialize simulations

T=12*100;
N=1000; % size of the population

% draw aggregate shocks
y=Y0*ones(1,T);
s=S0*ones(1,T);
rng(0,'twister');
for t=2:T
    [y(t),s(t)]=draw_ys(y(t-1),s(t-1),param,vars);
%     [y(t),~]=draw_ys(y(t-1),s(t-1),param,vars);
end




% declare variables

output=zeros(1,T);
outpp=zeros(1,T);
theta=zeros(1,T);
worker_earnings=zeros(N,T);
worker_z=zeros(N,T);
worker_n=zeros(N,T);
worker_nm1=zeros(N,T);
worker_status=zeros(N,T); % employed=1 or unemployed=0
worker_promised_utility=zeros(N,T); % employed=1 or unemployed=0
worker_exp_utility=zeros(N,T); % employed=1 or unemployed=0
UE=zeros(N,T); % indicator of a hire from unemployment
EU=zeros(N,T); % indicator of a layoff to unemployment
JJ=zeros(N,T); % indicator of a job-to-job transition
u_min=zeros(N,T); % keeps the min unemployment rate since beginning of the job
u_begin=zeros(N,T); % keeps the initial unemployment rate when job began
u_current=zeros(N,T); % keeps the current unemployment rate
tenure=zeros(N,T); % keeps the worker's tenure at the same job

% --------------------
% Simulate the economy


y0=y(1);
s0=s(1);
u0=U0;
g0=G0;

z0=zeros(N,1);
n0=zeros(N,1);
earnings0=zeros(N,1);
promised_utility0=zeros(N,1);
status1=zeros(N,1);
z1=zeros(N,1);
n1=zeros(N,1);
earnings1=zeros(N,1);
tenure1=zeros(N,1);


status0 = drawbinary((1-u0)*ones(N,1)); % initialize worker status

Ie=find(status0==1);
[z0(Ie),n0(Ie)] = draw_firmdist(param,vars,g0,length(Ie));
promised_utility0(Ie)=promised_utility(y0 +(s0-1)*param.ny + (z0(Ie)-1)*param.ny*param.ns + (n0(Ie)-1)*param.ny*param.ns*param.nz);

Iu=find(status0==0);
earnings0(Iu)=param.b;


fprintf('\nSimulate...\n');

for t=1:T
    y1=y(t);
    s1=s(t);
    
    [u1,g1,outpp(t),~,theta(t),~,~,~,entry,~,~,~,~,~,output(t),error] = distdyn(u0,g0,y1,s1,s0,param,vars,res);
    
    gpos=g0;
    gpos(gpos<0)=0;
    
    % evolution of unemployed workers
    
    n1=ones(N,1);
    promised_utility1=zeros(N,1);
    exp_utility1=zeros(N,1);
    
    Iu=find(status0==0);
    status1(Iu)=drawbinary(p(res.gtheta_xu(y1,s1),param)*ones(length(Iu),1));
    Iuu=find(status1==0 & status0==0);
    earnings1(Iuu)=param.b;
    
    Iue=find(status1==1 & status0==0);
    [z1(Iue),n0(Iue)]=draw_hiringdist(param,vars,res,y1,s1,gpos,length(Iue));
    n1(Iue)=res.gn_id(y1 +(s1-1)*param.ny + (z1(Iue)-1)*param.ny*param.ns + (n0(Iue)-1)*param.ny*param.ns*param.nz);
    promised_utility1(Iue)=res.gxu(y1,s1)*ones(length(Iue),1);
    exp_utility1(Iue)=worker_expected_utility(y1 +(s1-1)*param.ny + (z1(Iue)-1)*param.ny*param.ns + (n1(Iue)-1)*param.ny*param.ns*param.nz);
    earnings1(Iue)=promised_utility1(Iue)-exp_utility1(Iue);
    
    % evolution of employed workers
    
    Ie=find(status0==1);
    z1(Ie)=draw_z(s0,z0(Ie),param,vars);
    
    d=res.ge(y1 +(s1-1)*param.ny + (z1(Ie)-1)*param.ny*param.ns + (n0(Ie)-1)*param.ny*param.ns*param.nz);
    tau=res.gtau(y1 +(s1-1)*param.ny + (z1(Ie)-1)*param.ny*param.ns + (n0(Ie)-1)*param.ny*param.ns*param.nz);
    firing_prob=d+(1-d).*tau;
    status1(Ie)=drawbinary(1-firing_prob);    
    Ieu=find(status0==1 & status1==0);
    earnings1(Ieu)=param.b;
    
    
    Iee=find(status0==1 & status1==1);
    thetax=res.gtheta_x(y1 +(s1-1)*param.ny + (z1(Iee)-1)*param.ny*param.ns + (n0(Iee)-1)*param.ny*param.ns*param.nz);
    prob_jj=param.lambda*p(thetax,param);
    jj_vec=drawbinary(prob_jj);
   
    Inojj=Iee(find(jj_vec==0)); % guys that don't change firms    
    n1(Inojj)=res.gn_id(y1 +(s1-1)*param.ny + (z1(Inojj)-1)*param.ny*param.ns + (n0(Inojj)-1)*param.ny*param.ns*param.nz);
    promised_utility1(Inojj)=promised_utility(y1 +(s1-1)*param.ny + (z1(Inojj)-1)*param.ny*param.ns + (n0(Inojj)-1)*param.ny*param.ns*param.nz);
    exp_utility1(Inojj)=worker_expected_utility(y1 +(s1-1)*param.ny + (z1(Inojj)-1)*param.ny*param.ns + (n1(Inojj)-1)*param.ny*param.ns*param.nz);
    earnings1(Inojj)=promised_utility1(Inojj)-exp_utility1(Inojj);
    
 
    Ijj=Iee(find(jj_vec==1)); % guys that do a job-to-job transition    
    promised_utility1(Ijj)=res.gx(y1 +(s1-1)*param.ny + (z1(Ijj)-1)*param.ny*param.ns + (n0(Ijj)-1)*param.ny*param.ns*param.nz);
    [z1(Ijj),n0(Ijj)]=draw_hiringdist(param,vars,res,y1,s1,gpos,length(Ijj));  
    n1(Ijj)=res.gn_id(y1 +(s1-1)*param.ny + (z1(Ijj)-1)*param.ny*param.ns + (n0(Ijj)-1)*param.ny*param.ns*param.nz);
    exp_utility1(Ijj)=worker_expected_utility(y1 +(s1-1)*param.ny + (z1(Ijj)-1)*param.ny*param.ns + (n1(Ijj)-1)*param.ny*param.ns*param.nz);
    earnings1(Ijj)=promised_utility1(Ijj)-exp_utility1(Ijj);

    
    % update
    
    worker_earnings(:,t) = earnings1;
    worker_z(:,t) = z1;
    worker_n(:,t) = n1;
    worker_nm1(:,t) = n0;
    worker_promised_utility(:,t) = promised_utility1;
    worker_exp_utility(:,t) = exp_utility1;
    worker_status(:,t) = status1;
    
    
    z0 = z1;
    n0 = n1;
    earnings0 = earnings1;
    status0 = status1;
    
    
    UE(Iue,t)=1;  
    EU(Ieu,t)=1;
    JJ(Ijj,t)=1;
   
    tenure1=tenure1+1;    
    tenure1(Iu)=0;
    tenure1(Ijj)=0;
    tenure(:,t)=tenure1;
    
    
    for k=1:N
        if worker_status(k,t)==1
            if UE(k,t)==1 || JJ(k,t)==1 || t==1% job just started
                u_begin(k,t)=u1;
                u_min(k,t)=u1;
                u_max(k,t)=u1;
            else
                u_begin(k,t)=u_begin(k,t-1); 
                u_min(k,t)=min(u1,u_min(k,t-1));
                u_max(k,t)=max(u1,u_max(k,t-1));
            end                
        end
        u_current(k,t)=u1;
    end
    
    
    % Update state variables
    g0=g1;
    u0=u1;
    s0=s1;
end

% Prepare data

vect_wage=[];
vect_u_min=[];
vect_u_begin=[];
vect_u_current=[];
vect_outpp=[];
vect_y=[];
vect_s=[];
vect_tenure=[];

for t=1:T
    Ie=find(worker_status(:,t)==1);
    vect_wage = [vect_wage;worker_earnings(Ie,t)];
    vect_u_min = [vect_u_min;u_min(Ie,t)];
    vect_u_begin = [vect_u_begin;u_begin(Ie,t)];
    vect_u_current = [vect_u_current;u_current(Ie,t)];
    vect_outpp = [vect_outpp;outpp(t)*ones(length(Ie),1)];
    vect_y = [vect_y;vars.y_grid(y(t))*ones(length(Ie),1)];
    vect_s = [vect_s;vars.s_grid(s(t))*ones(length(Ie),1)];
    vect_tenure = [vect_tenure;tenure(Ie,t)];
end



% Return results

[X,STDX] = lscov([ones(length(vect_wage),1) vect_u_current],log(vect_wage));
fprintf('1st regression: log(wage) on u_current:\n');
X'
STDX'

[X,STDX] = lscov([ones(length(vect_wage),1) vect_u_begin],log(vect_wage));
fprintf('2nd regression: log(wage) on u_begin:\n');
X'
STDX'

[X,STDX] = lscov([ones(length(vect_wage),1) vect_u_min],log(vect_wage));
fprintf('3rd regression: log(wage) on u_min:\n');
X'
STDX'

[X,STDX] = lscov([ones(length(vect_wage),1) vect_u_current vect_u_begin vect_u_min],log(vect_wage));
fprintf('4th regression (all together): log(wage) on cons,u_current,u_begin,u_min:\n');
X'
STDX'

[X,STDX] = lscov([ones(length(vect_wage),1) vect_u_current vect_y vect_s],log(vect_wage));
fprintf('5th regression: log(wage) on u_current,y,s:\n');
X'
STDX'

[X,STDX] = lscov([ones(length(vect_wage),1) vect_u_begin vect_y vect_s vect_outpp],log(vect_wage));
fprintf('6th regression: log(wage) on u_begin,y,s,outpp:\n');
X'
STDX'

[X,STDX] = lscov([ones(length(vect_wage),1) vect_u_min vect_y vect_s vect_outpp min(vect_tenure,12)],log(vect_wage));
fprintf('7th regression: log(wage) on u_min,y,s,outpp,tenure:\n');
X'
STDX'

end

% -----------------------------------------------------------------
% findW.m: solves for the promised utility \hat{W}(y,s) that makes 
% workers choose the right gx for a firm of type (z,n)
%  
function [W] = findW(y,s,z,n,param,vars,res)
% returns W s.t argmax p(x')(x'-W) = gx
% since gx is known, solving for FOC: W=gx+p(theta(gx))/(pprime(theta(gx))*thetaprime(x))
W = res.gx(y,s,z,n)+p(res.gtheta_x(y,s,z,n),param)./(pprime(res.gtheta_x(y,s,z,n),param).*thetaxprime(res.gx(y,s,z,n),param,res.cost(y+(s-1)*param.ny)));
end

% thetaxprime.m: derivative of theta(y,s,x) as a function of x
function res = thetaxprime(x,param,cost)
res=1./qprime(qinv(param.c./(cost-x),param),param).*param.c./(cost-x).^2;
end

% drawbinary: draws a table of binary variables {0,1} with probability p. 
% p is table that contains the probability that each variable is equal to 1.
function vec = drawbinary(p)
Sz = size(p); 

vec = rand(Sz);

vec = vec <= p;
end

% draw_firmdist: draws N workers from firm distribution g.
function [z,n] = draw_firmdist(param,vars,g,N)
z = zeros(N,1);
n = zeros(N,1);

[z_id,n_id] = ndgrid(1:param.nz,1:param.nn);
n_dist=vars.n_grid(n_id).*g;
employment = sum (n_dist(:));

vec = employment*rand(N,1);
cdf=cumsum(n_dist(:));
[~,bin] = histc(vec,cdf);

z = z_id(bin);
n = n_id(bin);

end

% draw_hiringdist: draws N workers from the hiring distribution.
function [z,n] = draw_hiringdist(param,vars,res,y,s,g,N)
z = zeros(N,1);
n = zeros(N,1);

[z_id,n_id] = ndgrid(1:param.nz,1:param.nn);
ne_dist=squeeze(res.gne(y,s,:,:)).*g;
hiring = sum (ne_dist(:));

vec = hiring*rand(N,1);
cdf=cumsum(ne_dist(:));
[~,bin] = histc(vec,cdf);

z = z_id(bin);
n = n_id(bin);

end